/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/ssl.h: SSL support header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: ssl.h,v 1.6 2002/10/28 22:54:23 rageircd Exp $
 */

#ifndef __ssl_include__
#define __ssl_include__

#include "setup.h"

#ifdef HAVE_OPENSSL_ON

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <openssl/rand.h>
#include <openssl/err.h>
#include <openssl/bn.h>
#include <openssl/dh.h>
#include <openssl/crypto.h>
#include <openssl/rsa.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/evp.h>
#include <openssl/md5.h>
#include <openssl/blowfish.h>

#define SAFE_SSL_READ			1
#define SAFE_SSL_WRITE			2
#define SAFE_SSL_ACCEPT			3

#define SSL_ERR_CTX_NEW		0
#define SSL_ERR_CTX_USE_CERT 	-1
#define SSL_ERR_CTX_USE_KEY	-2
#define SSL_ERR_CTX_CHECK_KEY	-3

#define KEY_BITS 512
#define RAND_BITS KEY_BITS
#define RAND_BYTES (RAND_BITS / 8)
#define RAND_BYTES_HEX ((RAND_BYTES * 2) + 1)

#define PRIME_BITS 1024
#define PRIME_BYTES (PRIME_BITS / 8)
#define PRIME_BYTES_HEX ((PRIME_BYTES * 2) + 1)

typedef struct session_info SessionInfo;
typedef struct rc4_state RC4_state;
typedef unsigned char RC4_byte;
typedef unsigned int RC4_word;

struct session_info {
	DH *dh;
	char *session_shared;
	int session_shared_length;
};

struct rc4_state {
	RC4_byte mstate[256];
	RC4_byte x;
	RC4_byte y;
};

extern int init_dh();
extern void dh_end_session(void *);
extern void *dh_start_session();
extern char *dh_get_s_public(char *, int, void *);
extern int dh_get_s_shared(char *, int *, void *);
extern int dh_generate_shared(void *, char *);
extern int dh_hexstr_to_raw(char *, unsigned char *, int *);
extern void rc4_process_stream_to_buf(void *, const unsigned char *, unsigned char *, unsigned int);
extern void rc4_process_stream(void *, unsigned char *, unsigned int);
extern void *rc4_initstate(unsigned char *, int);
extern void rc4_destroystate(void *);

extern char *init_ssl();
extern char *ssl_get_cipher_info(SSL *);
extern int SSL_smart_shutdown(SSL *);

#endif

#endif /* __ssl_include__ */
