/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/struct.h: structures header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: struct.h,v 1.140 2002/12/15 21:26:16 rageircd Exp $
 */

#ifndef	__struct_include__
#define __struct_include__

#include "config.h"
#ifndef CONFIG_H_LEVEL_13
#error Incorrect config.h for this revision of ircd.
#endif

#include "conf2.h"
#include "queue.h"
#include "setup.h"
#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#ifdef HAVE_STDDEF_H
#include <stddef.h>
#endif
#ifdef ORATIMING
#include <sys/time.h>
#endif
#include "hash.h"
#include "ssl.h"
#include "../adns/adns.h"

typedef struct Client aClient;
typedef struct Channel aChannel;
typedef struct User anUser;
typedef struct Server aServer;
typedef struct SLink Link;
typedef struct channelMember chanMember;
typedef struct SMode Mode;
typedef struct Watch aWatch;
typedef struct Ban aBan;
typedef struct ListOptions LOpts;
typedef struct SearchOptions SOpts;
typedef struct Command aCommand;
typedef struct Whowas aWhowas;
typedef struct Ctab aCtab;
typedef struct ip_entry IP_ENTRY;
typedef struct module_require ModuleR;
typedef struct module_table ModuleT;
typedef struct module Module;
typedef struct event Event;
typedef struct hook Hook;
typedef struct scache_entry SCACHE;
#ifdef HAVE_THROTTLE_ON
typedef struct hashent_t hashent;
typedef struct hashent_list_t hashent_list;
typedef struct hash_table_t hash_table;
typedef struct throttle_t throttle;
#endif
typedef struct errmsg_t ErrorMsg;
typedef struct header_message_t HeaderMessage;
typedef struct listener Listener;
typedef struct dns_query DNSQuery;
typedef struct connauth ConnectAuth;
typedef struct message_file MessageFile;
typedef struct message_file_line MessageFileLine;

typedef long ts_val;

#include "dbuf.h"		/* THIS REALLY SHOULDN'T BE HERE!!! --msa */

#define	HOSTLEN			63	/* Length of hostname.  Updated to comply with RFC1123 */
#define HOSTIPLEN		15	/* Length of dotted quad form of IP */
#define	NICKLEN			30	/* Length of nickname */
#define MAX_DATE_STRING		32	/* Length of date string */
#define	USERLEN			10	/* Length of username */
#define	REALLEN			50	/* Length of realname */
#define	TOPICLEN		307	/* Length of topic */
#define	KILLLEN			400	/* Length of kill */
#define	CHANNELLEN		32	/* Length of channel name */
#define	PASSWDLEN		63	/* Length of passwords */
#define	KEYLEN			23	/* Length of channel key */
#define	BUFSIZE			512	/* Length of internal buffers */
#define	MAXRECIPIENTS		20
#define	MAXBANS			45
#define	MAXSILES		5
#define MAXSILELENGTH		128
#define MAXTARGETS		10
#define MAXDCCALLOW		5
#define MAXACCEPTS		10
#define MAXWHOREPLIES		200
#define MAXEVENTS		50
#define MAXHOOKTYPES		5
#define MAXWATCH		128
#define MAXKILLS		20
#define LOADCFREQ		5	/* HTM: every 5s */
#define LOADRECV		40	/* HTM: 40k/s */
#define LUSERS_CACHE_TIME	180
#define SCACHE_HASH_SIZE	257	/* SCACHE */
#define	USERHOST_REPLYLEN	(NICKLEN + HOSTLEN + USERLEN + 5)

#define DCC_LINK_ME		0x01	/* This is my dcc allow */
#define DCC_LINK_REMOTE		0x02    /* I need to remove these dcc allows from these clients when I die */

#define ACCEPT_LINK_ME		0x01	/* This is my message accept list */
#define ACCEPT_LINK_REMOTE	0x02	/* I need to remove these accpets from these clients when I die */

#ifdef HAVE_THROTTLE_ON
#define HASH_FL_NOCASE	0x1
#define HASH_FL_STRING	0x2
#endif

/* 'offsetof' is defined in ANSI-C. The following definition
 * is not absolutely portable (I have been told), but so far
 * it has worked on all machines I have needed it. The type
 * should be size_t but... --msa
 */

#ifndef offsetof
#define	offsetof(t,m) (int)((&((t *)0L)->m))
#endif

#define	elementsof(x) (sizeof(x)/sizeof(x[0]))
#define WHO_HOPCOUNT(s, a) ((IsULine((a)) && !HasMode((s), UMODE_OPER)) ? 0 : a->hopcount)

#define	STAT_LOG		-6
#define	STAT_MASTER		-5
#define	STAT_CONNECTING		-4
#define	STAT_HANDSHAKE		-3
#define	STAT_ME			-2
#define	STAT_UNKNOWN		-1
#define	STAT_SERVER		0
#define	STAT_CLIENT		1

/* status macros. */

#define	IsRegisteredUser(x)	((x)->status == STAT_CLIENT)
#define	IsRegistered(x)		((x)->status >= STAT_SERVER)
#define	IsConnecting(x)		((x)->status == STAT_CONNECTING)
#define	IsHandshake(x)		((x)->status == STAT_HANDSHAKE)
#define	IsMe(x)			((x)->status == STAT_ME)
#define	IsUnknown(x)		((x)->status == STAT_UNKNOWN || (x)->status == STAT_MASTER)
#define	IsServer(x)		((x)->status == STAT_SERVER)
#define	IsClient(x)		((x)->status == STAT_CLIENT)
#define	IsLog(x)		((x)->status == STAT_LOG)

#define	SetMaster(x)		((x)->status = STAT_MASTER)
#define	SetConnecting(x)	((x)->status = STAT_CONNECTING)
#define	SetHandshake(x)		((x)->status = STAT_HANDSHAKE)
#define	SetMe(x)		((x)->status = STAT_ME)
#define	SetUnknown(x)		((x)->status = STAT_UNKNOWN)
#define	SetServer(x)		((x)->status = STAT_SERVER)
#define	SetClient(x)		((x)->status = STAT_CLIENT)
#define	SetLog(x)		((x)->status = STAT_LOG)

#define	FLAGS_PINGSENT		0x00000001	/* Unreplied ping sent */
#define	FLAGS_DEADSOCKET	0x00000002	/* Local socket is dead--Exiting soon */
#define	FLAGS_KILLED		0x00000004	/* Prevents "QUIT" from being sent for this */
#define	FLAGS_BLOCKED		0x00000008	/* socket is in a blocked condition */
#define	FLAGS_CLOSING		0x00000010	/* set when closing to suppress errors */
#define	FLAGS_LOCAL		0x00000020	/* set for local clients */
#define	FLAGS_GOTID		0x00000040	/* successful ident lookup achieved */
#define	FLAGS_NONL		0x00000080	/* No \n in buffer */
#define FLAGS_NORMALEX		0x00000100	/* Client exited normally */
#define FLAGS_SENDQEX		0x00000200	/* Sendq exceeded */
#define FLAGS_ULINE		0x00000400	/* client is U-lined */
#define FLAGS_SECURE		0x00000800	/* client is secure */
#define FLAGS_HASHED_IP		0x00001000	/* client is in the ip address hash table */
#define FLAGS_HASHED		0x00002000	/* client is in the client/server hash table */

/* User Flags */
#define UFLAGS_DCCNOTICE	0x00000001	/* seen DCCALLOW notice */
#define UFLAGS_VHOST		0x00000002	/* client successfully logged into VHOST */

/* Server Flags */
#define SFLAGS_USERBURST	0x00000001	/* server in nick/server netburst */
#define SFLAGS_TOPICBURST	0x00000002	/* server in topic netburst */
#define SFLAGS_BURST		(SFLAGS_USERBURST|SFLAGS_TOPICBURST)
#define SFLAGS_SOBSENT		0x00000004	/* we've sent SOB */
#define SFLAGS_EOBRECV		0x00000008	/* we're waiting for EOB */
#define SFLAGS_SENDBURST	0x00000010	/* we're in the process of sending a burst */
#define SFLAGS_RC4_IN		0x00000020	/* link is encrypted */
#define SFLAGS_RC4_OUT		0x00000040	/* link is encrypted */
#define SFLAGS_RC4_NEG		0x00000080	/* server is in the process of negotiating (DKEY) */
#define SFLAGS_ZIP_IN		0x00000100	/* in data will be compressed */
#define SFLAGS_ZIP_OUT		0x00000200	/* out data is to be compressed */
#define SFLAGS_MAP		0x00000400	/* display server in MAP */

/* ConnectAuth Flags */
#define CAF_ID_CONNECTING	0x01
#define CAF_ID_PENDING		0x02
#define CAF_DNS_PENDING		0x04

/* ConnectAuth header messages */
#define CONNAUTH_DO_DNS		0
#define CONNAUTH_FIN_DNS	1
#define CONNAUTH_FAIL_DNS	2
#define CONNAUTH_DO_ID		3
#define CONNAUTH_FIN_ID		4
#define CONNAUTH_FAIL_ID	5
#define CONNAUTH_HOST_TOOLONG	6

#ifndef STATIC_MODULES
#define MOD_CALLTYPE_LOAD	0
#define MOD_CALLTYPE_RELOAD	1
#define MOD_CALLTYPE_UNLOAD	2
#endif

/* DKEY */
#define DKEY_GOTIN		0x01
#define DKEY_GOTOUT		0x02
#define DKEY_DONE(x)		(((x) & (DKEY_GOTIN|DKEY_GOTOUT)) == (DKEY_GOTIN|DKEY_GOTOUT))

#define CAPAB_NOQUIT		0x00000001	/* NOQUIT */
#define CAPAB_BURST		0x00000002	/* BURST */
#define CAPAB_UNCONN		0x00000004	/* UNCONNECT */
#define CAPAB_ZIP		0x00000008	/* ZIP */
#define CAPAB_DOZIP		0x00000010	/* output to link shall be zipped */
#define CAPAB_DKEY		0x00000020	/* DKEY */
#define CAPAB_DODKEY		0x00000040	/* output to link shall be encrypted */
#define CAPAB_SSJOIN		0x00000080	/* SSJOIN */
#define CAPAB_SSJv2		0x00000100	/* SSJv2 */
#define CAPAB_NICKIP		0x00000200	/* NICKIP */
#define CAPAB_VHOST		0x00000400	/* VHOST */
#define CAPAB_SNICK		0x00000800	/* SNICK */

#define HasCapab(x, y)		((x)->capab & (y))
#define AddCapab(x, y)		((x)->capab |= (y))
#define DelCapab(x, y)		((x)->capab &= ~(y))

/* User Modes */
#define UMODE_OPER		0x00000001	/* o: Operator */
#define UMODE_INVISIBLE		0x00000002	/* i: Invisible */
#define UMODE_SNOTICE		0x00000004	/* s: Server Notices */
#define UMODE_KNOTICE		0x00000008	/* k: Kill Notices */
#define UMODE_FNOTICE		0x00000010	/* f: Flood Notices */
#define UMODE_DNOTICE		0x00000020	/* D: Debug Notices */
#define UMODE_SPAMBOT		0x00000040	/* m: SpamBot Notices */
#define UMODE_DCCNOTICE		0x00000080	/* e: DCC Send Notices */
#define UMODE_LCLICONN		0x00000100	/* c: Local Client Connections */
#define UMODE_GCLICONN		0x00000200	/* G: Global Client Connections */
#define UMODE_WALLOPS		0x00000400	/* w: WALLOPS Notices */
#define UMODE_GLOBOPS		0x00000800	/* g: GLOBOPS Notices */
#define UMODE_CHATOPS		0x00001000	/* b: CHATOPS Notices */
#define UMODE_REGNICK		0x00002000	/* r: Registered Nickname */
#define UMODE_SPYING		0x00004000	/* y: Spying */
#define UMODE_SCLIENT		0x00008000	/* S: Services Client */
#define UMODE_SADMIN		0x00010000	/* a: Services Administrator */
#define UMODE_NETADMIN		0x00020000	/* N: Network Administrator */
#define UMODE_ADMIN		0x00040000	/* A: Server Administrator */
#define UMODE_COADMIN		0x00080000	/* C: Server Co-Administrator */
#define UMODE_HELPER		0x00100000	/* h: Helper */
#define UMODE_MASKED		0x00200000	/* z: Masked */
#define UMODE_PRIVACY		0x00400000	/* p: Privacy */
#define UMODE_DEAF		0x00800000	/* d: Deaf */
#define UMODE_RSTAFF		0x01000000	/* R: Routing Staff */

#define HasMode(x, y)		((x)->umode & (y))
#define AddMode(x, y)		((x)->umode |= (y))
#define DelMode(x, y)		((x)->umode &= ~(y))

#define MaskedHost(x)		(((x)->umode & UMODE_MASKED) ? (x)->user->maskedhost : (x)->host)

#define CCONN_LEV	1
#define REJ_LEV		2
#define SKILL_LEV	3
#define SPY_LEV		4
#define DEBUG_LEV	5
#define FLOOD_LEV 	6
#define SPAM_LEV 	7
#define DCCSEND_LEV	8
#define GCCONN_LEV	9

#define SEND_UMODES	(UMODE_OPER|UMODE_INVISIBLE|UMODE_WALLOPS|UMODE_REGNICK|UMODE_SADMIN|UMODE_NETADMIN| \
				UMODE_ADMIN|UMODE_COADMIN|UMODE_GCLICONN|UMODE_CHATOPS|UMODE_SCLIENT|UMODE_HELPER| \
				UMODE_MASKED|UMODE_PRIVACY|UMODE_DEAF)
#define ALL_UMODES	(UMODE_OPER|UMODE_INVISIBLE|UMODE_SNOTICE|UMODE_KNOTICE|UMODE_FNOTICE|UMODE_DNOTICE| \
				UMODE_SPAMBOT|UMODE_DCCNOTICE|UMODE_LCLICONN|UMODE_GCLICONN|UMODE_WALLOPS| \
				UMODE_GLOBOPS|UMODE_CHATOPS|UMODE_REGNICK|UMODE_SPYING|UMODE_SCLIENT|UMODE_SADMIN| \
				UMODE_NETADMIN|UMODE_ADMIN|UMODE_COADMIN|UMODE_HELPER|UMODE_MASKED|UMODE_PRIVACY| \
				UMODE_DEAF|UMODE_RSTAFF)
#define OPER_UMODES	(UMODE_OPER|UMODE_SNOTICE|UMODE_KNOTICE|UMODE_FNOTICE|UMODE_SPAMBOT|UMODE_DCCNOTICE| \
				UMODE_LCLICONN|UMODE_WALLOPS|UMODE_GLOBOPS|UMODE_SPYING)

#define	IsPerson(x)		((x)->user && IsClient(x))
#define	IsPrivileged(x)		((x)->umode & UMODE_OPER || IsServer(x))
#define	IsLocal(x)		((x)->flags & FLAGS_LOCAL)
#define	IsDead(x)		((x)->flags & FLAGS_DEADSOCKET)
#define	NoNewLine(x)		((x)->flags & FLAGS_NONL)
#define IsULine(x)		((x)->flags & FLAGS_ULINE)

#define OFLAG_DIE		0x00000001	/* oper can use DIE command */
#define OFLAG_RESTART		0x00000002	/* oper can use RESTART command */
#define OFLAG_REHASH		0x00000004	/* oper can use REHASH command */
#define OFLAG_KLINE		0x00000008	/* oper can use KLINE command */
#define OFLAG_UNKLINE		0x00000010	/* oper can use UNKLINE command */
#define OFLAG_ZAP		0x00000020	/* oper can use ZLINE command */
#define OFLAG_LROUTE		0x00000040	/* oper can do local routing */
#define OFLAG_GROUTE		0x00000080	/* oper can do global routing */
#define OFLAG_LKILL		0x00000100	/* oper can do local kills */
#define OFLAG_GKILL		0x00000200	/* oper can do global kills */
#define OFLAG_LOCOPS		0x00000400	/* oper can send LOCOPS notices */
#define OFLAG_GLOBOPS		0x00000800	/* oper can send GLOBOPS notices */
#define OFLAG_WALLOPS		0x00001000	/* oper can send WALLOPS notices */
#define OFLAG_LNOTICE		0x00002000	/* oper can send local notices */
#define OFLAG_GNOTICE		0x00004000	/* oper can send global notices */
#define OFLAG_FNOTICE		0x00008000	/* oper can receive flood notices */
#define OFLAG_LCLICONN		0x00010000	/* oper can receive local cliconnect notices */
#define OFLAG_GCLICONN		0x00020000	/* oper can receive global cliconnect notices */
#define OFLAG_NETADMIN		0x00040000	/* oper is a Network Administrator */
#define OFLAG_ADMIN		0x00080000	/* oper is a Server Administrator */
#define OFLAG_COADMIN		0x00100000	/* oper is a Server Co-Administrator */
#define OFLAG_SADMIN		0x00200000	/* oper is a Services Administrator */
#define OFLAG_RSTAFF		0x00400000	/* oper is routing staff (can see routing notices and server ip's) */
#define OFLAG_OPERMOTD		0x00800000	/* oper receives OPERMOTD on oper-up */
#define OFLAG_AUTOMASK		0x01000000	/* oper receives new maskedhost on oper-up */

#define OPHasFlag(x, y)		((x)->oflag & (y))
#define OPAddFlag(x, y)		((x)->oflag |= (y))
#define OPDelFlag(x, y)		((x)->oflag &= ~(y))

/* defined debugging levels */
#define	DEBUG_FATAL  0
#define	DEBUG_ERROR  1 /* report_error() and other errors that are found */
#define	DEBUG_NOTICE 3
#define	DEBUG_DNS    4 /* used by all DNS related routines - a *lot* */
#define	DEBUG_INFO   5 /* general usful info */
#define	DEBUG_NUM    6 /* numerics */
#define	DEBUG_SEND   7 /* everything that is sent out */
#define	DEBUG_DEBUG  8 /* anything to do with debugging, ie unimportant :) */
#define	DEBUG_MALLOC 9 /* malloc/free calls */
#define	DEBUG_LIST  10 /* debug list use */

/* defines for curses in client */
#define	DUMMY_TERM	0
#define	CURSES_TERM	1
#define	TERMCAP_TERM	2

/* Hook definitions */
#define HOOKTYPE_JOIN	0

/* MessageFile types */
#define MFTYPE_MOTD	0
#define MFTYPE_OPERMOTD	1
#define MFTYPE_RULES	2

struct Counter {
	int server;		/* servers */
	int myserver;		/* my servers */
	int myulined;		/* my ulined servers */
	int oper;		/* opers */
	int chan;		/* channels */
	int local;		/* local clients */
	int total;		/* total clients */
	int invisi;		/* invisible clients */
	int unknown;		/* unknown connections */
	int max_loc;		/* max local clients */
	int max_tot;		/* max global clients */
	int cli_restart;	/* clients since (re)start */
	ts_val start;		/* when we started collecting info */
	u_long today;		/* client connections today */
	ts_val day;		/* when today started */
	u_long weekly;		/* connections this week */
	ts_val week;		/* when this week started */
	u_long monthly;		/* connections this month */
	ts_val month;		/* when this month started */
	u_long yearly;		/* connections this year (this is gunna be be big!) */
	ts_val year;		/* when this year started */
};

struct Command {
	aCommand *next, *prev;
	char *cmd;
	int (*func)();
	unsigned int count;
	int paras;
	unsigned unreg_use : 1;
	unsigned reset_idle : 1;
	unsigned long bytes;
};

struct Whowas {
	int hashv;
	char name[NICKLEN + 1];
	char username[USERLEN + 1];
	char hostname[HOSTLEN + 1];
	char maskedhost[HOSTLEN + 1];
	char realname[REALLEN + 1];
	char *servername;
	time_t logoff;
	aClient *online;
	aWhowas *next, *prev;
	aWhowas *cnext, *cprev;
};

/* channel mode and flag tab structure */
struct Ctab {
	u_int mode;
	char flag;
	unsigned halfop : 1;
	unsigned param : 1;
};

/* iphash entry structure */
struct ip_entry {
	unsigned long ip;
	int count;
	IP_ENTRY *next;
};

/* module table structure */
struct module_table {
	ModuleT *next, *prev;
	Module *header;
};

/* module require */
struct module_require {
	unsigned commands : 1;
	unsigned m_join : 1;
	unsigned m_kick : 1;
	unsigned m_message : 1;
	unsigned m_mode : 1;
	unsigned m_nick : 1;
	unsigned m_part : 1;
	unsigned m_ping : 1;
	unsigned m_pong : 1;
	unsigned m_quit : 1;
	unsigned m_user : 1;
};

/* module structure */
struct module {
	char *name;
	char *info;
	int version;
	char *revision;
	unsigned core_module : 1;
	void *ptr;
	void (*unloadfunc)();
};

/* event structure */
struct event {
	const char *name;
	void (*func)();
	void *arg;
	time_t frequency;
	time_t when;
	Event *next;
	unsigned active : 1;
	unsigned loop : 1;
};

/* hook structure */
struct hook {
	int (*ifunc)();
	void (*vfunc)();
	Hook *next, *prev;
};

/* sache structure */
struct scache_entry {
	char name[HOSTLEN + 1];
	SCACHE *next;
};

#ifdef HAVE_THROTTLE_ON
struct hashent_t {
	void *ent;
	SLIST_ENTRY(hashent_t) lp;
};

struct hash_table_t {
	int size;
	hashent_list *table;
	size_t keyoffset;
	size_t keylen;
	int flags;
	int (*cmpfunc)(void *, void *);
};

struct throttle_t {
	char addr[HOSTIPLEN + 1];
	int conns;
	time_t first;
	time_t last;
	time_t zline_start;
	int stage;
	int re_zlines;
	LIST_ENTRY(throttle_t) lp;
};
#endif

struct errmsg_t {
	int numeric;
	char *message;
};

struct header_message_t {
	const char *string;
	size_t length;
};

struct listener {
	Listener *next;
	char *ip;
	int port;
	unsigned int options;
	int fd;
	u_long count;
	struct in_addr ipaddr;
	time_t lasttime;
	long sendM;
	long receiveM;
	long sendK;
	long receiveK;
	u_short sendB;
	u_short receiveB;
	ConfigItem_bind *bind;
};

struct dns_query {
	void *ptr;
	adns_query query;
	adns_answer answer;
	void (*callback)();
};

struct connauth {
	ConnectAuth *next, *prev;
	aClient *cptr;
	unsigned int flags;
	int fd;
	time_t timeout;
};

struct message_file {
	char *filename;
	short type;
	MessageFileLine *data;
	char changed_date[MAX_DATE_STRING + 1];
};

struct message_file_line {
	MessageFileLine *next;
	char *line;
};

/* Client structures */
struct User {
	char *maskedhost;
	chanMember *channel;
	char *away;
	int joined;
	char *server;
	unsigned int servicestamp;
	Link *silence;
        Link *dccallow;
	Link *accept;
	time_t last_accept_notice;
	time_t first_message_time;
	int number_of_messages;
	int flood_notice;
	Link *watch;
	int watches;
	aWhowas *whowas;
	unsigned int uflags;

	/* WARNING! Local clients only below this - do NOT move last ! */
	time_t last;
	Link *invited;
	LOpts *lopt;
	time_t last_nick_change;
	int nick_changes;
	time_t last_away_change;
	int away_changes;
};

struct Server {
	char *up;
	char *connected_by;
	int dkey_flags;			/* dkey flags */
#ifdef HAVE_OPENSSL_ON
	void *sessioninfo_in;		/* pointer to opaque sessioninfo structure */
	void *sessioninfo_out;		/* pointer to opaque sessioninfo structure */
	void *rc4_in;			/* etc */
	void *rc4_out;			/* etc */
#endif
	void *zip_out;			/* inbound zipped data */
	void *zip_in;			/* outbound zipped data */
	ConfigItem_link *conf;		/* config */
	unsigned int sflags;		/* server flags */
};

struct Client {
	struct Client *next, *prev, *hnext;
	anUser *user;
	aServer *serv;
	time_t lasttime;
	time_t firsttime;
	time_t since;
	ts_val tsinfo;
	unsigned long flags;
	unsigned long umode;
	aClient *from;
	aClient *uplink;
	int fd;
  	int hopcount;
	short status;
	char nicksent;
	char name[HOSTLEN + 1];
	char username[USERLEN + 1];
	char host[HOSTLEN + 1];
	char info[REALLEN + 1];
#ifdef FLUD
	Link *fludees;
#endif
	struct in_addr ip;
	char hostip[HOSTIPLEN + 1];

	/* WARNING! Local clients only below this - do NOT move count ! */
#ifdef HAVE_OPENSSL_ON
	SSL *ssl;
#endif
	int count;
#ifdef FLUD
	time_t fludblock;
	struct fludbot *fluders;
#endif
	time_t last_join_time;
	time_t last_leave_time;
	int join_leave_count;
	int oper_warn_count_down;
	char buffer[BUFSIZE];
	short lastsq;
	DBuf sendQ;
	DBuf recvQ;
	long sendM;
	long sendK;
	long receiveM;
	long receiveK;
	u_short sendB;
	u_short receiveB;
	long lastrecvM;
	int priority;
  	Link *confs;
	char sockhost[HOSTLEN + 1];
	char passwd[PASSWDLEN + 1];
	unsigned short port;
	DNSQuery *dnsquery;
	int sockerr;
	int pingval;
	int sendqlen;
	long capab;
	long oflag;
	Listener *listener;
	ConfigItem_class *class;
};

#define	CLIENT_LOCAL_SIZE sizeof(aClient)
#define	CLIENT_REMOTE_SIZE offsetof(aClient, count)
#define USER_LOCAL_SIZE sizeof(anUser)
#define USER_REMOTE_SIZE offsetof(anUser, last)

/* statistics structures */
struct stats {
	unsigned int is_cl;     /* number of client connections */
	unsigned int is_sv;     /* number of server connections */
	unsigned int is_ni;     /* connection but no idea who it was */
	unsigned short is_cbs;  /* bytes sent to clients */
	unsigned short is_cbr;  /* bytes received to clients */
	unsigned short is_sbs;	/* bytes sent to servers */
	unsigned short is_sbr;  /* bytes received to servers */
	unsigned long is_cks;	  /* k-bytes sent to clients */
	unsigned long is_ckr;	  /* k-bytes received to clients */
	unsigned long is_sks;	  /* k-bytes sent to servers */
	unsigned long is_skr;	  /* k-bytes received to servers */
	time_t      is_cti;		  /* time spent connected by clients */
	time_t      is_sti;		  /* time spent connected by servers */
	unsigned int is_ac;		  /* connections accepted */
	unsigned int is_ref;	  /* accepts refused */
	unsigned int is_unco;	  /* unknown commands */
	unsigned int is_wrdi;	  /* command going in wrong direction */
	unsigned int is_unpf;	  /* unknown prefix */
	unsigned int is_empt;	  /* empty message */
	unsigned int is_num;	  /* numeric message */
	unsigned int is_kill;  	/* number of kills generated on collisions */
	unsigned int is_fake;	  /* MODE 'fakes' */
	unsigned int is_asuc;	  /* successful auth requests */
	unsigned int is_abad;	  /* bad auth requests */
	unsigned int is_udp;		/* packets recv'd on udp port */
	unsigned int is_loc;		/* local connections made */
#ifdef FLUD
	unsigned int is_flud;	  /* users/channels flood protected */
#endif	                  /* FLUD */
};

/* mode structure for channels */
struct SMode {
	unsigned int mode;
	int limit;
	char key[KEYLEN + 1];
};

struct Ban {
	char *banstr;
	char *who;
	time_t when;
	aBan *next;
};

/* channel member link structure, used for chanmember chains */
struct channelMember {
	chanMember *nextuser;
	chanMember *nextchan;
        aClient *cptr;
	aChannel *chptr;
        int flags;
	int bans;
};

/* general link structure used for chains */

struct SLink {
	struct SLink *next;
	int flags;
	union {
		aClient *cptr;
		aChannel *chptr;
		ConfigItem *conf;
		aBan *banptr;
		aWatch *wptr;
		char *cp;
	} value;
};

/* channel structure */

struct Channel {
	struct Channel *nextch, *prevch, *hnextch;
	int hashv;
	Mode mode;
	char topic[TOPICLEN + 1];
        char topic_nick[NICKLEN + 1];
	time_t topic_time;
	int users;
	chanMember *members;
	Link *invites;
	aBan *banlist;
	aBan *exbanlist;
	aBan *invexlist;
	ts_val channelts;
#ifdef FLUD
	time_t fludblock;
	struct fludbot *fluders;
#endif
	char chname[CHANNELLEN+1];
	time_t last_knock;
	time_t first_message_time;
	int number_of_messages;
	int flood_notice;
};

#define TS_CURRENT	5		/* Current TS protocol version */
#define TS_MIN		3		/* Minimum TS protocol version */
#define TS_DOESTS	0x20000000
#define DoesTS(x)	((x)->tsinfo == TS_DOESTS)

#define CHFL_CHANADMIN		0x0001	/* Channel Administrator */
#define	CHFL_CHANOP		0x0002	/* Channel Operator */
#define CHFL_HALFOP		0x0004	/* Channel Half Operator */
#define	CHFL_VOICE		0x0008	/* the power to speak */
#define	CHFL_DEOPPED		0x0010	/* deopped by us, modes need to be bounced */

#define MODE_CHANADMIN		CHFL_CHANADMIN
#define	MODE_CHANOP		CHFL_CHANOP
#define MODE_HALFOP		CHFL_HALFOP
#define	MODE_VOICE		CHFL_VOICE
#define	MODE_DEOPPED		CHFL_DEOPPED
#define	MODE_PRIVATE		0x0020
#define	MODE_SECRET		0x0040
#define	MODE_MODERATED		0x0080
#define	MODE_TOPICLIMIT		0x0100
#define	MODE_INVITEONLY		0x0200
#define	MODE_NOPRIVMSGS		0x0400
#define	MODE_KEY		0x0800
#define	MODE_BAN		0x1000
#define	MODE_LIMIT		0x2000
#define MODE_REGISTERED		0x4000
#define MODE_REGONLY		0x8000
#define MODE_NOCOLOUR		0x10000
#define MODE_OPERONLY		0x20000
#define MODE_ADMINONLY		0x40000
#define MODE_STRIPCOLOUR	0x80000
#define MODE_EXBAN		0x100000
#define MODE_NOCTCP		0x200000
#define MODE_NONICKCH		0x400000
#define MODE_INVEX		0x800000

/* mode flags which take another parameter (With PARAmeterS) */
#define	MODE_WPARAS	(MODE_CHANOP|MODE_VOICE|MODE_BAN|MODE_KEY|MODE_LIMIT|MODE_HALFOP|\
			MODE_CHANADMIN|MODE_EXBAN|MODE_INVEX)

#define	HoldChannel(x)		(!(x))
#define	SecretChannel(x)	((x) && ((x)->mode.mode & MODE_SECRET))
#define	ShowChannel(v,c)	(PubChannel(c) || IsMember((v),(c)))
#define	PubChannel(x)		((!x) || ((x)->mode.mode & MODE_SECRET) == 0)
#define IsMember(blah, chan)	((blah && blah->user && find_user_member((blah->user)->channel, chan)) ? 1 : 0)
#define	IsChannelName(name)	((name) && (*(name) == '#' || *(name) == '&'))

/* Misc macros */

#define	BadPtr(x)	(!(x) || (*(x) == '\0'))
#define	isvalid(c)	(((c) >= 'A' && (c) < '~') || isdigit(c) || (c) == '-')
#define	MyConnect(x)	((x)->fd >= 0)
#define	MyClient(x)	(MyConnect(x) && IsClient(x))
#define	MyOper(x)	(MyConnect(x) && HasMode((x), UMODE_OPER))
#define IsSendable(x)	(DBufLength(&x->sendQ) < 16384)
#define DoList(x)	(((x)->user) && ((x)->user->lopt))
#define strncpyzt(x, y, N)	do { strncpy(x, y, N); x[N - 1] = '\0'; } while (0)

#define	MODE_ADD       0x40000000
#define	MODE_DEL       0x20000000

/* return values for hunt_server() */

#define	HUNTED_NOSUCH	(-1)	/* if the hunted server is not found */
#define	HUNTED_ISME	0	/* if this server should execute the command */
#define	HUNTED_PASS	1	/* if message passed onwards successfully */

/* used when sending to #mask or $mask */

#define	MATCH_SERVER  1
#define	MATCH_HOST    2

/* misc variable externs */

extern char ircd_version[128], *build;
extern char *infotext[], *dalinfotext[];
extern char *generation, *creation;

/* misc defines */

#define ZIP_NEXT_BUFFER -4
#define RC4_NEXT_BUFFER -3
#define	FLUSH_BUFFER	-2

/* client authentication return values */

#define CLIENTAUTH_MATCHED	0	/* access granted (successfully matched I line) */
#define CLIENTAUTH_NOMATCH	-1	/* access denied (failed to match an I line) */
#define CLIENTAUTH_SOCKERR	-2	/* socket error */
#define CLIENTAUTH_ILINEFULL	-3	/* I line is full */
#define CLIENTAUTH_INVALIDPW	-4	/* Invalid password */
#define CLIENTAUTH_TOOMANYIPS	-5	/* Too many connections from a single ip address */
#define CLIENTAUTH_ERRMSG	"Unauthorized client connection from %s%s"

#ifdef ORATIMING

/*
 * Timing stuff (for performance measurements): compile with
 * -DORATIMING and put a TMRESET where you want the counter of time
 * spent set to 0, a TMPRINT where you want the accumulated results,
 * and TMYES/TMNO pairs around the parts you want timed -orabidoo
 */

extern struct timeval tsdnow, tsdthen;
extern unsigned long tsdms;

#define TMRESET	tsdms = 0;
#define TMYES	gettimeofday(&tsdthen, NULL);
#define TMNO	gettimeofday(&tsdnow, NULL); if (tsdnow.tv_sec != tsdthen.tv_sec) tsdms += 1000000 * (tsdnow.tv_sec - tsdthen.tv_sec); tsdms += tsdnow.tv_usec; tsdms -= tsdthen.tv_usec;
#define TMPRINT	sendto_ops("Time spent: %ld ms", tsdms);
#else
#define TMRESET
#define TMYES
#define TMNO
#define TMPRINT
#endif

#ifdef FLUD
struct fludbot {
	struct Client *fluder;
	int         count;
	time_t      first_msg, last_msg;
	struct fludbot *next;
};

#endif /* FLUD */

struct Watch {
	   aWatch  *hnext;
	   time_t   lasttime;
	   Link  *watch;
	   char  nick[1];
};

struct ListOptions {
	   LOpts *next;
	   Link  *yeslist, *nolist;
	   int   starthash;
	   short int   showall;
	   unsigned short usermin;
	   int   usermax;
	   time_t   currenttime;
	   time_t   chantimemin;
	   time_t   chantimemax;
	   time_t   topictimemin;
	   time_t   topictimemax;
};

struct SearchOptions {
	int umodes;
	char *nick;
	char *user;
	char *host;
	char *ip;
	char *gcos;
	aChannel *channel;
	aClient *server;
	unsigned umode_plus : 1;
	unsigned nick_plus : 1;
	unsigned user_plus : 1;
	unsigned host_plus : 1;
	unsigned ip_plus : 1;
	unsigned gcos_plus : 1;
	unsigned chan_plus : 1;
	unsigned serv_plus : 1;
	unsigned away_plus : 1;
	unsigned check_away : 1;
	unsigned check_umode: 1;
	unsigned show_chan : 1;
	unsigned search_chan : 1;
};

/* internal defines for cptr->sockerr */
#define IRCERR_BUFALLOC		-11
#define IRCERR_ZIP		-12
#define IRCERR_SSL		-13

#define runHook4(hooktype, a, b, c, d)	for (ghook = HookTable[hooktype]; ghook != NULL; ghook = ghook->next) { \
					if (ghook->ifunc != NULL) ghook->ifunc(a, b, c, d); \
					if (ghook->vfunc !=NULL) ghook->vfunc(a, b, c, d); \
				}

#endif /* __struct_include__ */
