/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/aliases.c: services' aliases module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: aliases.c,v 1.17 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/aliases.so",
	"Services' Aliases",
	4,
	"$Revision: 1.17 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("CHANSERV",	m_chanserv, 1, MSG_NONE);
	addCommand("CS",	m_chanserv, 1, MSG_NONE);
	addCommand("NICKSERV",	m_nickserv, 1, MSG_NONE);
	addCommand("NS",	m_nickserv, 1, MSG_NONE);
	addCommand("MEMOSERV",	m_memoserv, 1, MSG_NONE);
	addCommand("MS",	m_memoserv, 1, MSG_NONE);
	addCommand("OPERSERV",	m_operserv, 1, MSG_NONE);
	addCommand("OS",	m_operserv, 1, MSG_NONE);
	addCommand("STATSERV",	m_statserv, 1, MSG_NONE);
	addCommand("SS",	m_statserv, 1, MSG_NONE);
	addCommand("SERVICES",	m_services, 1, MSG_NONE);
	addCommand("IDENTIFY",	m_identify, 1, MSG_NONE);
}
void _module_unload() {
	delCommand("CHANSERV",	m_chanserv);
	delCommand("CS",	m_chanserv);
	delCommand("NICKSERV",	m_nickserv);
	delCommand("NS",	m_nickserv);
	delCommand("MEMOSERV",	m_memoserv);
	delCommand("MS",	m_memoserv);
	delCommand("OPERSERV",	m_operserv);
	delCommand("OS",	m_operserv);
	delCommand("STATSERV",	m_statserv);
	delCommand("SS",	m_statserv);
	delCommand("SERVICES",	m_services);
	delCommand("IDENTIFY",	m_identify);
}
#endif

static int handle_alias(aClient *cptr, int parc, char *parv[], char *client_server,
  char *client_nick, char *func_name)
{
	aClient *acptr = NULL;

	if (BadPtr(client_server)) {
		sendto_one(cptr, err_str(ERR_FUNCDISABLED), me.name, cptr->name, func_name);
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(cptr, err_str(ERR_NOTEXTTOSEND), me.name, cptr->name);
		return 0;
	}
	if ((acptr = find_person(client_nick, NULL)) == NULL) {
		sendto_one(cptr, err_str(ERR_SERVICESDOWN), me.name, cptr->name, client_nick);
		return 0;
	}
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", cptr->name, client_nick, client_server, parv[1]);
	return 0;
}

/*
 * m_chanserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_chanserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return handle_alias(cptr, parc, parv, SERVICES_SERVER, CHANSERV, "CHANSERV");
}

/*
 * m_nickserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_nickserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return handle_alias(cptr, parc, parv, SERVICES_SERVER, NICKSERV, "NICKSERV");
}

/*
 * m_memoserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_memoserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return handle_alias(cptr, parc, parv, SERVICES_SERVER, MEMOSERV, "MEMOSERV");
}

/*
 * m_operserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_operserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return handle_alias(cptr, parc, parv, SERVICES_SERVER, OPERSERV, "OPERSERV");
}

/*
 * m_statserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_statserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return handle_alias(cptr, parc, parv, STATS_SERVER, STATSERV, "STATSERV");
}

/*
 * m_services
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_services(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *tmp = NULL;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
		return 0;
	}
	if (BadPtr(SERVICES_SERVER)) {
		sendto_one(sptr, err_str(ERR_FUNCDISABLED), me.name, parv[0], "SERVICES");
		return 0;
	}
	if (!mycmp(parv[1], "help")) {
		sendto_one(sptr, ":%s NOTICE %s :For ChanServ help, use /chanserv help", me.name, parv[0]);
		sendto_one(sptr, ":%s NOTICE %s :For NickServ help, use /nickserv help", me.name, parv[0]);
		sendto_one(sptr, ":%s NOTICE %s :For Memoserv help, use /memoserv help", me.name, parv[0]);
		return 0;
	}
	if ((tmp = strchr(parv[1], ' ')) != NULL) {
		while (*tmp == ' ') {
			tmp++;
		}
		return (*tmp == '#') ? m_chanserv(cptr, sptr, parc, parv) :  m_nickserv(cptr, sptr, parc, parv);
	}
	return m_nickserv(cptr, sptr, parc, parv);
}

/*
 * m_identify
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_identify(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	int gate = 0;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
		return 0;
	}
	if (BadPtr(SERVICES_SERVER)) {
		sendto_one(sptr, err_str(ERR_FUNCDISABLED), me.name, parv[0], "IDENTIFY");
		return 0;
	}
	if (*parv[1] == '#' && strchr(parv[1], ' ') != NULL) {
		acptr = find_person(CHANSERV, NULL);
		gate = 1;
	}
	else {
		acptr = find_person(NICKSERV, NULL);
		gate = 0;
	}
	if (acptr == NULL) {
		sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0],
			gate ? CHANSERV : NICKSERV);
		return 0;
	}
	sendto_one(acptr, ":%s PRIVMSG %s@%s :IDENTIFY %s", parv[0],
		gate ? CHANSERV : NICKSERV, SERVICES_SERVER, parv[1]);
	return 0;
}
