/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_away.c: AWAY command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_away.c,v 1.14 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_away.so",
	"/AWAY command",
	4,
	"$Revision: 1.14 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("AWAY", m_away, MAXPARA, MSG_RESETIDLE);
	return;
}
void _module_unload() {
	delCommand("AWAY", m_away);
	return;
}
#endif

/*
 * m_away
 *	parv[0] = sender prefix
 *	parv[1] = reason
 */
int m_away(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *reason = (parc > 1 && *parv[1] != '\0') ? parv[1] : NULL;

	if (sptr->user == NULL) {
		return 0;
	}
	if (ANTIAWAYFLOOD == 1 && MyClient(sptr)) {
		if (sptr->user->last_away_change + MAX_AWAY_TIME < timeofday) {
			sptr->user->away_changes = 0;
		}
		sptr->user->last_away_change = timeofday;
		sptr->user->away_changes++;
	}
	if (parc < 2 || reason == NULL) {
		if (sptr->user->away != NULL) {
			MyFree(sptr->user->away);
			sptr->user->away = NULL;
			sendto_serv_butone(cptr, ":%s AWAY", parv[0]);
		}
		if (MyClient(sptr)) {
			sendto_one(sptr, rpl_str(RPL_UNAWAY), me.name, parv[0]);
		}
		return 0;
	}
	if (ANTIAWAYFLOOD == 1 && MyClient(sptr) && (sptr->user->away_changes > MAX_AWAY_COUNT) &&
	  !HasMode(sptr, UMODE_OPER)) {
		sendto_one(sptr, err_str(ERR_TOOMANYAWAY), me.name, parv[0]);
		return 0;
	}
	if (strlen(reason) > TOPICLEN) {
		reason[TOPICLEN] = '\0';
	}

	sendto_serv_butone(cptr, ":%s AWAY :%s", parv[0], parv[1]);
	if (sptr->user->away != NULL) {
		MyFree(sptr->user->away);
	}
	DupString(sptr->user->away, reason);

	if (MyClient(sptr)) {
		sendto_one(sptr, rpl_str(RPL_NOWAWAY), me.name, parv[0]);
	}
	return 0;
}
