/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_ison.c: ISON command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_ison.c,v 1.10 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>  
#include <string.h>
#include <fcntl.h>

static char buf[BUFSIZE];

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_ison.so",
	"/ISON command",
	4,
	"$Revision: 1.10 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("ISON", m_ison, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("ISON", m_ison);
	return;
}
#endif

/*
 * m_ison
 *	parv[0] = sender prefix
 *	parv[1] = nickname list
 */
int m_ison(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *s, **pav = parv, *p = NULL;
	int len, len2;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "ISON");
		return 0;
	}

	ircsprintf(buf, rpl_str(RPL_ISON), me.name, parv[0]);
	len = strlen(buf);

	if (!HasMode(cptr, UMODE_OPER)) {
		cptr->priority += 20;
	}

	for (s = strtoken(&p, *++pav, " "); s; s = strtoken(&p, NULL, " ")) {
		acptr = find_person(s, NULL);
		if (acptr) {
			len2 = strlen(acptr->name);
			if (len + len2 + 5 < sizeof(buf)) {
				strcat(buf, acptr->name);
				len += len2;
				strcat(buf, " ");
				len++;
			}
			else {
				break;
			}
		}
	}
	sendto_one(sptr, "%s", buf);
	return 0;
}
