/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_kick.c: KICK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_kick.c,v 1.13 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_kick.so",
	"/KICK command",
	4,
	"$Revision: 1.13 $",
	1, NULL, NULL
};
void _module_load() {
	addCommand("KICK", m_kick, MAXPARA, MSG_NONE);
	RequiredModules.m_kick = 1;
	return;
}
void _module_unload() {
	delCommand("KICK", m_kick);
	RequiredModules.m_kick = 0;
	return;
}
#endif

/*
 * m_kick
 *	parv[0] = sender prefix
 *	parv[1] = channel list
 *	parv[2] = client list
 *	parv[3] = comment
 */
int m_kick(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	aChannel *chptr = NULL;
	char *name, *user, *p = NULL, *p2 = NULL;
	char *comment = (parc > 3 && *parv[3] != '\0') ? parv[3] : parv[0];
	int chasing = 0;

	if (parc < 3 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "KICK");
		return 0;
	}
	if (strlen(comment) > TOPICLEN) {
		comment[TOPICLEN] = '\0';
	}
	for (; (name = strtoken(&p, parv[1], ",")); parv[1] = NULL) {
		if ((chptr = find_channel(name, NULL)) == NULL) {
			sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0], name);
			continue;
		}
		if (!IsServer(sptr) && !is_chan_op(sptr, chptr) && !is_half_op(sptr, chptr) &&
		  !IsULine(sptr) && !HasMode(sptr, UMODE_OPER)) {
			if (MyConnect(sptr)) {
				sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED), me.name, parv[0],
					chptr->chname);
				continue;
			}
			if (chptr->channelts == 0) {
				sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED), me.name, parv[0],
					chptr->chname);
				continue;
			}
		}

		for (; (user = strtoken(&p2, parv[2], ",")); parv[2] = NULL) {
			if ((acptr = find_chasing(sptr, user, &chasing)) == NULL) {
				continue;
			}
			if (IsMember(acptr, chptr)) {
				if (!IsServer(sptr) && !IsULine(sptr)) {
					if (!is_chan_admin(sptr, chptr) && is_chan_admin(acptr, chptr)) {
						sendto_one(sptr, ":%s NOTICE %s :*** Cannot kick %s - "
							"channel administrator", me.name, parv[0], user);
						continue;
					}
					if (is_half_op(sptr, chptr) && !is_chan_op(sptr, chptr) &&
					  is_chan_op(acptr, chptr)) {
						sendto_one(sptr, ":%s NOTICE %s :*** Cannot kick %s - "
						"you are only a halfop", me.name, parv[0], user);
						continue;
					}
				}
				sendto_channel_butserv(chptr, sptr, ":%s KICK %s %s :%s", parv[0], name,
					acptr->name, comment);
				sendto_match_servs(chptr, cptr, ":%s KICK %s %s :%s", parv[0], name,
					acptr->name, comment);
				remove_user_from_channel(acptr, chptr);
			}
			else {
				sendto_one(sptr, err_str(ERR_USERNOTINCHANNEL), me.name, parv[0], user, name);
			}
		}
	}
	return 0;
}
