/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_knock.c: KNOCK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_knock.c,v 1.15 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern void clean_channelname(unsigned char *);
extern int check_channelmask(aClient *, aClient *, char *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_knock.so",
	"/KNOCK command",
	4,
	"$Revision: 1.15 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("KNOCK", m_knock, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("KNOCK", m_knock);
	return;
}
#endif

/*
 * m_knock
 *	parv[0] = sender prefix
 *	parv[1] = channel name
 *	parv[2] = optional channel key
 */
int m_knock(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	char *p, *name, *key;

	if (ALLOWKNOCKS == 0) {
		sendto_one(sptr, rpl_str(ERR_FUNCDISABLED), me.name, parv[0], "KNOCK");
		return 0;
	}
	if (parc < 2 || BadPtr(parv[1])) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "KNOCK");
		return 0;
	}
	if (MyConnect(sptr)) {
		clean_channelname(parv[1]);
	}
	if (check_channelmask(sptr, cptr, parv[1])) {
		return 0;
	}

	name = parv[1];
	key = (parc > 2 && *parv[2] != '\0') ? parv[2] : NULL;
	if ((p = strchr(name, ',')) != NULL) {
		*p = '\0';
	}
	if (!IsChannelName(name) || (chptr = find_channel(name, NULL)) == NULL) {
		sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0], name);
		return 0;
	}
	if (IsMember(sptr, chptr)) {
		sendto_one(sptr, rpl_str(RPL_KNOCKONCHAN), me.name, parv[0], chptr->chname);
		return 0;
	}
	if (!(chptr->mode.mode & MODE_INVITEONLY) && ((chptr->mode.limit == 0 || (chptr->mode.limit != 0
	  && chptr->users < chptr->mode.limit)) || ((chptr->mode.mode & MODE_INVITEONLY) &&
	  has_invex(sptr, chptr)))) {
		sendto_one(sptr, rpl_str(RPL_KNOCKCHANOPEN), me.name, parv[0], chptr->chname);
		return 0;
	}
	if (is_banned(sptr, chptr)) {
		sendto_one(sptr, err_str(ERR_BANNEDFROMCHAN), me.name, parv[0], chptr->chname);
		return 0;
	}
	if (*chptr->mode.key != '\0' && (key == NULL || irccmp(chptr->mode.key, key))) {
		sendto_one(sptr, err_str(ERR_BADCHANNELKEY), me.name, parv[0], chptr->chname);
		return 0;
	}
	if (chptr->mode.mode & MODE_PRIVATE) {
		sendto_one(sptr, err_str(ERR_CANNOTSENDTOCHAN), me.name, parv[0], chptr->chname);
		return 0;
	}
	if (chptr->last_knock + KNOCKDELAY > timeofday) {
		sendto_one(sptr, rpl_str(RPL_KNOCKWAIT), me.name, parv[0], chptr->chname, 
			(int)KNOCKDELAY - (timeofday - chptr->last_knock));
		return 0;
	}
	chptr->last_knock = timeofday;
	sendto_channelflag_butone(CHFL_CHANOP|CHFL_HALFOP, cptr, sptr, chptr, rpl_str(RPL_KNOCK),
		me.name, chptr->chname, chptr->chname, parv[0], sptr->username, MaskedHost(sptr));
	sendto_one(sptr, rpl_str(RPL_KNOCKDELIVERED), me.name, parv[0], chptr->chname);
	return 0;
}
