/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_pong.c: PONG command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_pong.c,v 1.21 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern void send_topic_burst(aClient *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_pong.so",
	"/PONG command",
	4,
	"$Revision: 1.21 $",
	1, NULL, NULL
};
void _module_load() {
	addCommand("PONG", m_pong, MAXPARA, MSG_NONE);
	RequiredModules.m_pong = 1;
	return;
}
void _module_unload() {
	delCommand("PONG", m_pong);
	RequiredModules.m_pong = 0;
	return;
}
#endif

/*
 * m_pong
 *	parv[0] = sender prefix
 *	parv[1] = origin
 *	parv[2] = destination
 */
int m_pong(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *origin, *destination;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOORIGIN), me.name, parv[0]);
		return 0;
	}

	origin = parv[1];
	destination = parv[2];
	cptr->flags &= ~FLAGS_PINGSENT;
	sptr->flags &= ~FLAGS_PINGSENT;

	if (sptr == cptr && IsServer(cptr)) {
		if (sptr->serv->sflags & SFLAGS_USERBURST) {
			sptr->serv->sflags &= ~SFLAGS_USERBURST;
			sendto_gnotice("%s has processed user/channel burst, sending topic burst.",
				sptr->name);
			send_topic_burst(sptr);
			sptr->flags |= FLAGS_PINGSENT;
			sptr->serv->sflags |= SFLAGS_SOBSENT;
			sendto_one(sptr, "PING :%s", me.name);
		}
		else if (sptr->serv->sflags & SFLAGS_TOPICBURST) {
			char capablist[25];

			sptr->serv->sflags &= ~SFLAGS_TOPICBURST;
			ircsprintf(capablist, "%s%s%s",
				HasCapab(sptr, CAPAB_SNICK) ? " SNICK" : HasCapab(sptr, CAPAB_NICKIP) ? " NICKIP" : "",
				HasCapab(sptr, CAPAB_SSJv2) ? " SSJv2" : HasCapab(sptr, CAPAB_SSJOIN) ? " SSJOIN" : "",
				HasCapab(sptr, CAPAB_VHOST) ? " VHOST" : "");
			sendto_gnotice("%s has processed topic burst, synch complete. (capabilities:%s)",
				sptr->name, (*capablist == ' ') ? capablist : " NONE");
#ifdef HAVE_HUB_ON
			sendto_serv_butone(sptr, ":%s GNOTICE :%s has processed topic burst, synch "
				"complete. (capabilities:%s)", me.name, sptr->name,
				(*capablist == ' ') ? capablist : " NONE");
#endif
			sendto_one(sptr, "PING :%s", me.name);
		}
	}
	if (!BadPtr(destination) && mycmp(destination, me.name) && IsRegistered(sptr)) {
		if ((acptr = find_client(destination, NULL)) == NULL) {
			if ((acptr = find_server(destination, NULL)) == NULL) {
				sendto_one(sptr, err_str(ERR_NOSUCHSERVER), me.name, parv[0],
					destination);
				return 0;
			}
		}
		sendto_one(acptr, ":%s PONG %s %s", parv[0], origin, destination);
	}
#ifdef DEBUGMODE
	else {
		Debug((DEBUG_NOTICE, "PONG: %s %s", origin, destination ? destination : "*"));
	}
#endif
	return 0;
}
