/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_svsmode.c: SVSMODE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_svsmode.c,v 1.8 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern int user_modes[];

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_svsmode.so",
	"/SVSMODE command",
	4,
	"$Revision: 1.8 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("SVSMODE", m_svsmode, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("SVSMODE", m_svsmode);
	return;
}
#endif

/*
 * m_svsmode
 *	parv[0] = sender prefix
 *	parv[1] = nickname
 *	parv[2] = TS (or mode, depending on SVS version)
 *	parv[3] = mode (or serviceid, if old SVS version)
 *	parv[4] = optional argument (serviceid)
 */
int m_svsmode(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int flag, *s, what = MODE_ADD, setflags = 0;
	char *m, *modes, *optarg = NULL;
	aClient *acptr;
	ts_val ts = 0;

	if (!IsULine(sptr) || parc < 3) {
		return 0;
	}
	if (parc >= 4 && (parv[3][0] == '+' || parv[3][0] == '-')) {
		ts = atol(parv[2]);
		modes = parv[3];
		optarg = (parc > 4) ? parv[4] : NULL;
	}
	else {
		modes = parv[2];
		optarg = (parc > 3) ? parv[3] : NULL;
	}
	if ((acptr = find_person(parv[1], NULL)) == NULL) {
		return 0;
	}
	if (ts != 0 && ts != acptr->tsinfo) {
		return 0;
	}
	for (s = user_modes; (flag = *s); s += 2) {
		if (acptr->umode & flag) {
			setflags |= flag;
		}
	}
	for (m = modes; *m != '\0'; m++) {
		switch (*m) {
			case '+':
				what = MODE_ADD;
				break;
			case '-':
				what = MODE_DEL;
				break;
			case ' ':
			case '\n':
			case '\r':
			case '\t':
				break;
			case 'd':
				if (optarg && isdigit(*optarg)) {
					acptr->user->servicestamp = strtoul(optarg, NULL, 0);
				}
				break;
			default:
				for (s = user_modes; (flag = *s); s += 2) {
					if (*m == (char)(*(s + 1))) {
						if (what == MODE_ADD) {
							acptr->umode |= flag;
						}
						else {
							acptr->umode &= ~flag;
						}
						if (MyConnect(acptr) && what == MODE_DEL &&
						  flag == UMODE_OPER && !HasMode(acptr, UMODE_OPER)) {
							delfrom_fdlist(acptr->fd, &oper_fdlist);
						}
						break;
					}
				}
				break;
		}
	}
	if (optarg != NULL) {
		sendto_serv_butone(cptr, ":%s SVSMODE %s %ld %s %s", parv[0], parv[1],
			acptr->tsinfo, modes, optarg);
	}
	else {
		sendto_serv_butone(cptr, ":%s SVSMODE %s %ld %s", parv[0], parv[1],
			acptr->tsinfo, modes);
	}
	return 0;
}
