/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_szline.c: SZLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_szline.c,v 1.4 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_szline.so",
	"/SZLINE command",
	4,
	"$Revision: 1.4 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("SZLINE", m_szline, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("SZLINE", m_szline);
	return;
}
#endif

/*
 * m_szline
 *	parv[0] = sender prefix
 *	parv[1] = ip address
 *	parv[2] = reason
 */
int m_szline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	ConfigItem_ban *ban;
	char *reason = (parc > 2 && *parv[2] != '\0') ? parv[2] : "no reason";

	if (!(IsServer(sptr) || IsULine(sptr))) {
		return 0;
	}
	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "SQLINE");
		return 0;
	}
	if ((ban = find_ban(BAN_IP, parv[1])) == NULL) {
		ban = (ConfigItem_ban *)MyMallocX(sizeof(ConfigItem_ban));
		ban->type = (BAN_IP|BAN_IP_S);
		DupString(ban->mask, parv[1]);
		DupString(ban->reason, reason);
		ban->flag.temporary = 0;
		add_ConfigItem((ConfigItem *)ban, (ConfigItem **)&ci_ban);
	}
	sendto_serv_butone(cptr, ":%s SZLINE %s :%s", parv[0], parv[1], parv[2]);
	return 0;
}
