/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_trace.c: TRACE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_trace.c,v 1.18 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_trace.so",
	"/TRACE command",
	4,
	"$Revision: 1.18 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("TRACE", m_trace, MAXPARA, MSG_NONE);
	return;
}
void _module_unload(void) {
	delCommand("TRACE", m_trace);
	return;
}
#endif

/*
 * m_trace
 *	parv[0] = sender prefix
 *	parv[1] = server name
 */
int m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static time_t last_used = 0L;
	aClient *acptr = NULL;
	ConfigItem_class *class = NULL;
	int i, doall, link_s[MAXCONNECTIONS + 1], link_u[MAXCONNECTIONS + 1];
	int cnt = 0, wilds = 0, dow = 0;
	char *tname = parc > 1 ? parv[1] : me.name;

	if (HIDEULINES == 1) {
		if ((acptr = next_client_double(client, tname)) != NULL) {
			if (!HasMode(sptr, UMODE_OPER) && IsULine(acptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				return 0;
			}
			acptr = NULL;
		}
	}
	if (parc > 2) {
		if (hunt_server(cptr, sptr, ":%s TRACE %s :%s", 2, parc, parv)) {
			return 0;
		}
	}
	switch (hunt_server(cptr, sptr, ":%s TRACE :%s", 1, parc, parv)) {
		case HUNTED_PASS:
		{
			aClient *acptr2 = next_client_double(client, tname);
			sendto_one(sptr, rpl_str(RPL_TRACELINK), me.name, parv[0], ircd_version,
				debug_mode, tname,
				(acptr2->from != NULL && acptr2->from->name != NULL) ?
				acptr2->from->name : "acptr2_is_NULL!");
			return 0;
		}
		case HUNTED_ISME:
			break;
		default:
			return 0;
	}
	if (!HasMode(sptr, UMODE_OPER)) {
		if (parv[1] != NULL && strchr(parv[1], '.') == NULL && (strchr(parv[1], '*') != NULL
		  || strchr(parv[1], '?') != NULL)) {
			sendto_one(sptr, rpl_str(RPL_ENDOFTRACE), me.name, parv[0], parv[1]);
			return 0;
		}
		if (last_used + PACETIME_INTENSE > timeofday) {
			sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
			return 0;
		}
		else {
			last_used = timeofday;
		}
		if (SPYNOTICES == 1 && IsPerson(sptr)) {
			sendto_realops_lev(SPY_LEV, "TRACE requested by %s (%s@%s) [%s]", sptr->name,
				sptr->username, MaskedHost(sptr), sptr->user->server);
		}
	}

	doall = (parv[1] && parc > 1) ? !match(tname, me.name) : TRUE;
	wilds = !parv[1] || strchr(tname, '*') || strchr(tname, '?');
	dow = (wilds || doall);

	if (!HasMode(sptr, UMODE_OPER) || !dow) {
		char *name, *class;
		if ((acptr = hash_find_client(tname, (aClient *)NULL)) == NULL || !IsPerson(acptr)) {
			sendto_one(sptr, rpl_str(RPL_ENDOFTRACE), me.name, parv[0], tname);
			return 0;
		}
		name = get_client_name(acptr, HasMode(sptr, UMODE_RSTAFF) ? FALSE : HIDEME);
		class = acptr->class != NULL ? acptr->class->name : default_class->name;
		sendto_one(sptr, rpl_str(HasMode(acptr, UMODE_OPER) ? RPL_TRACEOPERATOR : RPL_TRACEUSER), me.name,
			parv[0], class, name, timeofday - acptr->lasttime);
		sendto_one(sptr, rpl_str(RPL_ENDOFTRACE), me.name, parv[0], tname);
		return 0;
	}
	if (dow == 1 && lifesux > 0 && !HasMode(sptr, UMODE_OPER)) {
		sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
		return 0;
	}

	memset((char *)link_s, '\0', sizeof(link_s));
	memset((char *)link_u, '\0', sizeof(link_u));
	if (doall == 1) {
		for (acptr = client; acptr != NULL; acptr = acptr->next) {
			if (IsPerson(acptr) && (!HasMode(acptr, UMODE_INVISIBLE) || HasMode(sptr, UMODE_OPER))) {
				link_u[(acptr->from->fd == me.fd) ? MAXCONNECTIONS : acptr->from->fd]++;
			}
			else if (IsServer(acptr)) {
				if (HIDEULINES == 1) {
					if (HasMode(sptr, UMODE_OPER) || !IsULine(acptr)) {
						link_s[(acptr->from->fd == me.fd) ? MAXCONNECTIONS : acptr->from->fd]++;
					}
				}
				else {
					link_s[(acptr->from->fd == me.fd) ? MAXCONNECTIONS : acptr->from->fd]++;
				}
			}
		}
	}
	for (i = 0; i <= highest_fd; i++) {
		char *name, *class;
		if (!(acptr = local[i])) {
			continue;
		}
		if (HIDEULINES == 1 && !HasMode(sptr, UMODE_OPER) && IsULine(acptr)) {
			continue;
		}
		if (HasMode(acptr, UMODE_INVISIBLE) && dow && !(MyConnect(sptr) && HasMode(sptr, UMODE_OPER)) &&
		  !HasMode(acptr, UMODE_OPER) && acptr != sptr) {
			continue;
		}
		if (dow == 0 && mycmp(tname, acptr->name)) {
			continue;
		}

		if (acptr->status == STAT_CLIENT || acptr->status == STAT_UNKNOWN) {
			name = get_client_name(acptr, HasMode(sptr, UMODE_OPER) ? FALSE : HIDEME);
		}
		else {
			name = get_client_name(acptr, HasMode(sptr, UMODE_RSTAFF) ? FALSE : HIDEME);
		}
		class = acptr->class != NULL ? acptr->class->name : default_class->name;
		switch (acptr->status) {
			case STAT_CONNECTING:
				sendto_one(sptr, rpl_str(RPL_TRACECONNECTING), me.name, parv[0],
					class, name);
				cnt++;
				break;
			case STAT_HANDSHAKE:
				sendto_one(sptr, rpl_str(RPL_TRACEHANDSHAKE), me.name, parv[0],
					class, name);
				cnt++;
				break;
			case STAT_ME:
				break;
			case STAT_UNKNOWN:
				sendto_one(sptr, rpl_str(RPL_TRACEUNKNOWN), me.name, parv[0],
					class, name,
					acptr->firsttime ? timeofday - acptr->firsttime : -1);
				cnt++;
				break;
			case STAT_CLIENT:
				if (((HasMode(sptr, UMODE_OPER) && MyClient(sptr)) || !(dow == 1 &&
				  HasMode(acptr, UMODE_INVISIBLE))) || dow == 0 || HasMode(acptr, UMODE_OPER)) {
					if (HasMode(acptr, UMODE_OPER)) {
						sendto_one(sptr, rpl_str(RPL_TRACEOPERATOR), me.name,
							parv[0], class, name, timeofday - acptr->lasttime);
					}
				}
				break;
			case STAT_SERVER:
				sendto_one(sptr, rpl_str(RPL_TRACESERVER), me.name, parv[0], class,
					link_s[i], link_u[i], name, acptr->serv->connected_by);
				cnt++;
				break;
			case STAT_LOG:
				sendto_one(sptr, rpl_str(RPL_TRACELOG), me.name, parv[0], LOGFILE,
					acptr->port);
				cnt++;
				break;
			default:
				sendto_one(sptr, rpl_str(RPL_TRACENEWTYPE), me.name, parv[0], name);
				cnt++;
				break;
		}
	}
	if (!OPHasFlag(sptr, OFLAG_WALLOPS) || cnt == 0) {
		if (cnt != 0) {
			sendto_one(sptr, rpl_str(RPL_ENDOFTRACE), me.name, parv[0], tname);
			return 0;
		}
		sendto_one(sptr, rpl_str(RPL_TRACESERVER), me.name, parv[0], 0, link_s[MAXCONNECTIONS],
			link_u[MAXCONNECTIONS], me.name, "*", "*", me.name, timeofday - acptr->lasttime);
		sendto_one(sptr, rpl_str(RPL_ENDOFTRACE), me.name, parv[0], tname);
		return 0;
	}
	if (HIDEULINES == 1 && HasMode(sptr, UMODE_OPER)) {
		for (class = ci_class; class != NULL; class = (ConfigItem_class *)class->next) {
			sendto_one(sptr, rpl_str(RPL_TRACECLASS), me.name, parv[0],
				class->name ? class->name : "<noname>", class->clients);
		}
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFTRACE), me.name, parv[0], tname);
	return 0;
}
