/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_version.c: VERSION command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_version.c,v 1.21 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "patchlevel.h"
#include "isupport.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern char server_options[];

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_version.so",
	"/VERSION command",
	4,
	"$Revision: 1.21 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("VERSION", m_version, MAXPARA, MSG_UNREGUSE);
	return;
}
void _module_unload() {
	delCommand("VERSION", m_version);
	return;
}
#endif

/*
 * m_version
 *	parv[0] = sender prefix
 *	parv[1] = remote server
 */
int m_version(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *seriesstatus = "UNKNOWN";
	static time_t last_used = 0L;

	if (!HasMode(sptr, UMODE_OPER)) {
		if (last_used + PACETIME_SIMPLE > timeofday) {
			sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
			return 0;
		}
		else {
			last_used = timeofday;
		}
		if (SPYNOTICES == 1 && IsPerson(sptr)) {
			sendto_realops_lev(SPY_LEV, "VERSION requested by %s (%s@%s) [%s]", sptr->name,
				sptr->username, MaskedHost(sptr), sptr->user->server);
		}
	}
	if (hunt_server(cptr, sptr, ":%s VERSION :%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}

#if SERIESSTATUS == DEVEL
	seriesstatus = "DEVEL";
#elif SERIESSTATUS == BETA
	seriesstatus = "BETA";
#elif SERIESSTATUS == RELEASE
	seriesstatus = "RELEASE";
#elif SERIESSTATUS == STABLE
	seriesstatus = "STABLE";
#endif
	sendto_one(sptr, rpl_str(RPL_VERSION), me.name, parv[0], ircd_version, build, me.name,
		server_options, TS_CURRENT, seriesstatus);
	sendto_one(sptr, rpl_str(RPL_ISUPPORT), me.name, parv[0], isupport_one);
	sendto_one(sptr, rpl_str(RPL_ISUPPORT), me.name, parv[0], isupport_two);
	return 0;
}
