/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_whowas.c: WHOWAS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_whowas.c,v 1.12 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern char *canonize(char *);
extern void init_whowas();
extern unsigned int hash_whowas_name(char *);
extern aWhowas WHOWAS[NICKNAMEHISTORYLENGTH];
extern aWhowas *WHOWASHASH[WW_MAX];

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_whowas.so",
	"/WHOWAS command",
	4,
	"$Revision: 1.12 $",
	0, NULL, NULL
};
void _module_load() {
	init_whowas();
	addCommand("WHOWAS", m_whowas, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("WHOWAS", m_whowas);
	return;
}
#endif

/*
 * m_whowas
 *	parv[0] = sender prefix
 *	parv[1] = nickname queried
 */
int m_whowas(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aWhowas *tmp = NULL;
	int cur = 0, found = 0, max = -1;
	char *p, *nick, *s;
	static time_t last_used = 0L;

	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "WHOWAS");
		return 0;
	}
	if (parc > 2) {
		max = atoi(parv[2]);
	}

	if (!HasMode(sptr, UMODE_OPER)) {
		if (last_used + PACETIME_SIMPLE > timeofday) {
			sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
			return 0;
		}
		else {
			last_used = timeofday;
		}
	}
	if (parc > 3) {
		if (hunt_server(cptr, sptr, ":%s WHOWAS %s %s :%s", 3, parc, parv)) {
			return 0;
		}
	}

	parv[1] = canonize(parv[1]);
	if (!MyConnect(sptr) && max > 20) {
		max = 20;
	}
	for (s = parv[1]; (nick = strtoken(&p, s, ",")) != NULL; s = NULL) {
		found = 0;
		for (tmp = WHOWASHASH[hash_whowas_name(nick)]; tmp != NULL; tmp = tmp->next) {
			if (!mycmp(nick, tmp->name)) {
				sendto_one(sptr, rpl_str(RPL_WHOWASUSER), me.name, parv[0],
					tmp->name, tmp->username, tmp->maskedhost, tmp->realname);
				if (HasMode(sptr, UMODE_OPER)) {
					sendto_one(sptr, rpl_str(RPL_WHOWASHOST), me.name, parv[0],
						tmp->name, tmp->hostname);
				}
				sendto_one(sptr, rpl_str(RPL_WHOISSERVER), me.name, parv[0],
					tmp->name, tmp->servername, myctime(tmp->logoff));
				cur++;
				found++;
			}
			if (max > 0 && cur > max) {
				break;
			}
		}
		if (found == 0) {
			sendto_one(sptr, err_str(ERR_WASNOSUCHNICK), me.name, parv[0], nick);
		}
		if (p != NULL) {
			p[-1] = ',';
		}
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFWHOWAS), me.name, parv[0], parv[1]);
	return 0;
}
