/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/channel.c: channel management
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: channel.c,v 1.75 2002/11/20 20:06:23 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "channel.h"
#include "h.h"
#include "memory.h"

aChannel *channel = NULL;
BlockHeap *channel_heap;
BlockHeap *chanmember_heap;
BlockHeap *channelban_heap;

static char buf[BUFSIZE];
static char modebuf[REALMODEBUFLEN];
static char parabuf[REALMODEBUFLEN];

void init_channels()
{
	channel_heap = BlockHeapCreate((size_t)sizeof(aChannel), CHANNELS_PREALLOCATE);
	chanmember_heap = BlockHeapCreate((size_t)sizeof(chanMember), CHANMEMBERS_PREALLOCATE);
	channelban_heap = BlockHeapCreate((size_t)sizeof(aBan), BANS_PREALLOCATE);
	return;
}

aClient *find_chasing(aClient *sptr, char *user, int *chasing)
{
	aClient *acptr = find_client(user, NULL);

	if (chasing) {
		*chasing = 0;
	}
	if (acptr !=  NULL) {
		return acptr;
	}
	if ((acptr = get_history(user, (long)KILLCHASETIMELIMIT)) == NULL) {
		if (!IsServer(sptr)) {
			sendto_one(sptr, err_str(ERR_NOSUCHNICK), me.name, sptr->name, user);
		}
		return NULL;
	}
	if (chasing) {
		*chasing = 1;
	}
	return acptr;
}

int add_ban_tolist(aClient *cptr, aChannel *chptr, char *banid, aBan **list, int bquiet)
{
	aBan *ban;
	int cnt = 0;
	chanMember *cm;

	for (ban = *list; ban != NULL; ban = ban->next) {
		if (MyClient(cptr)) {
			if (++cnt >= MAXBANS) {
				sendto_one(cptr, err_str(ERR_BANLISTFULL), me.name, cptr->name,
					chptr->chname, banid);
				return 0;
			}
			else if (!match(ban->banstr, banid)) {
				return 0;
			}
		}
		else if (!mycmp(ban->banstr, banid)) {
			return 0;
		}
	}

	ban = make_channelban();

	DupString(ban->banstr, banid);
	if (IsPerson(cptr)) {
		ban->who = (char *)MyMalloc(strlen(cptr->name) + strlen(cptr->username) +
		  strlen(MaskedHost(cptr)) + 3);
		ircsprintf(ban->who, "%s!%s@%s", cptr->name, cptr->username, MaskedHost(cptr));
	}
	else {
		DupString(ban->who, cptr->name);
	}
	ban->when = timeofday;
	ban->next = *list;
	*list = ban;

	if (bquiet == 1) {
		for (cm = chptr->members; cm != NULL; cm = cm->nextuser) {
			if (!MyConnect(cm->cptr)) {
				continue;
			}
			if (HasMode(cm->cptr, UMODE_MASKED) && irccmp(cm->cptr->user->maskedhost,
			  cm->cptr->host)) {
				if (!match(banid, make_nick_user_host(cm->cptr->name, cm->cptr->username,
				  cm->cptr->user->maskedhost))) {
					cm->bans++;
					continue;
				}
			}
			if (!match(banid, make_nick_user_host(cm->cptr->name, cm->cptr->username,
			  cm->cptr->host))) {
				cm->bans++;
				continue;
			}
			if (!match(banid, make_nick_user_host(cm->cptr->name, cm->cptr->username,
			  cm->cptr->hostip))) {
				cm->bans++;
				continue;
			}
		}
	}
	return 1;
}

int del_ban_fromlist(aChannel *chptr, char *banid, aBan **list, int bquiet)
{
	aBan **ban, *tmp;
	chanMember *cm;

	if (BadPtr(banid)) {
		return 0;
	}
	for (ban = &(*list); *ban != NULL; ban = &((*ban)->next)) {
		if (!mycmp(banid, (*ban)->banstr)) {
			tmp = *ban;
			*ban = tmp->next;
			MyFree(tmp->banstr);
			MyFree(tmp->who);
			free_channelban(tmp);

			if (bquiet == 1) {
				for (cm = chptr->members; cm != NULL; cm = cm->nextuser) {
					if (!MyConnect(cm->cptr) || cm->bans == 0) {
						continue;
					}
					if (HasMode(cm->cptr, UMODE_MASKED) &&
					  irccmp(cm->cptr->user->maskedhost, cm->cptr->host)) {
						if (!match(banid, make_nick_user_host(cm->cptr->name,
						  cm->cptr->username, cm->cptr->user->maskedhost))) {
							cm->bans--;
							continue;
						}
					}
					if (!match(banid, make_nick_user_host(cm->cptr->name,
					  cm->cptr->username, cm->cptr->host))) {
						cm->bans--;
						continue;
					}
					if (!match(banid, make_nick_user_host(cm->cptr->name,
					  cm->cptr->username, cm->cptr->hostip))) {
						cm->bans--;
						continue;
					}
				}
			}
			return 1;
		}
	}
	return 0;
}

int is_banned(aClient *cptr, aChannel *chptr)
{
	int found_exception = 0, masked = 0;
	aBan *ban, *exban;
	char host[NICKLEN + USERLEN + HOSTLEN + 6], maskedhost[NICKLEN + USERLEN + HOSTLEN + 6];
	char *ip;

	if (!IsPerson(cptr)) {
		return 0;
	}
	if (HasMode(cptr, UMODE_MASKED) && irccmp(cptr->host, cptr->user->maskedhost)) {
		strcpy(maskedhost, make_nick_user_host(cptr->name, cptr->username, cptr->user->maskedhost));
		masked = 1;
	}
	strcpy(host, make_nick_user_host(cptr->name, cptr->username, cptr->host));
	ip = make_nick_user_host(cptr->name, cptr->username, cptr->hostip);

	for (ban = chptr->banlist; ban != NULL; ban = ban->next) {
		found_exception = 0;
		if ((masked == 1 && !match(ban->banstr, maskedhost)) || !match(ban->banstr, host) ||
		  !match(ban->banstr, ip)) {
			for (exban = chptr->exbanlist; exban != NULL; exban = exban->next) {
				if ((masked == 1 && !match(exban->banstr, maskedhost)) ||
				  !match(exban->banstr, host) || !match(exban->banstr, ip)) {
					found_exception = 1;
					break;
				}
			}
			if (found_exception == 1) {
				continue;
			}
			return 1;
		}
	}
	return 0;
}

int has_invex(aClient *cptr, aChannel *chptr)
{
	aBan *invex;
	char host[NICKLEN + USERLEN + HOSTLEN + 6], maskedhost[NICKLEN + USERLEN + HOSTLEN + 6];
	char *ip;
	int masked = 0;

	if (!IsPerson(cptr)) {
		return 0;
	}
	if (HasMode(cptr, UMODE_MASKED) && irccmp(cptr->host, cptr->user->maskedhost)) {
		strcpy(maskedhost, make_nick_user_host(cptr->name, cptr->username, cptr->user->maskedhost));
		masked = 1;
	}
	strcpy(host, make_nick_user_host(cptr->name, cptr->username, cptr->host));
	ip = make_nick_user_host(cptr->name, cptr->username, cptr->hostip);

	for (invex = chptr->invexlist; invex != NULL; invex = invex->next) {
		if ((masked == 1 && !match(invex->banstr, maskedhost)) || !match(invex->banstr, host) ||
		  !match(invex->banstr, ip)) {
			return 1;
		}
	}
	return 0;
}

chanMember *add_user_to_channel(aChannel *chptr, aClient *cptr, int flags)
{
	chanMember *cm;

	if (cptr->user != NULL) {
		cm = make_chanmember();
		cm->flags = flags;
		cm->cptr = cptr;
		cm->chptr = chptr;
		cm->nextuser = chptr->members;
		cm->nextchan = cptr->user->channel;
		cm->bans = 0;

		chptr->members = cm;
		chptr->users++;

		cptr->user->channel = cm;
		cptr->user->joined++;
		return cm;
	}
	return NULL;
}

static void sub1_from_channel(aChannel *chptr)
{
	Link *tmp;
	aBan *bp, *next;

	if (--chptr->users <= 0) {
		while ((tmp = chptr->invites) != NULL) {
			del_invite(tmp->value.cptr, chptr);
		}
		for (bp = chptr->banlist; bp != NULL; bp = next) {
			next = bp->next;
			MyFree(bp->banstr);
			MyFree(bp->who);
			free_channelban(bp);
		}
		for (bp = chptr->exbanlist; bp != NULL; bp = next) {
			next = bp->next;
			MyFree(bp->banstr);
			MyFree(bp->who);
			free_channelban(bp);
		}
		for (bp = chptr->invexlist; bp != NULL; bp = next) {
			next = bp->next;
			MyFree(bp->banstr);
			MyFree(bp->who);
			free_channelban(bp);
		}
		if (chptr->prevch != NULL) {
			chptr->prevch->nextch = chptr->nextch;
		}
		else {
			channel = chptr->nextch;
		}
		if (chptr->nextch != NULL) {
			chptr->nextch->prevch = chptr->prevch;
		}
		del_from_channel_hash_table(chptr->chname, chptr);
#ifdef FLUD
		free_fluders(NULL, chptr);
#endif
		free_channel(chptr);
		Count.chan--;
	}
	return;
}

chanMember *remove_user_from_channel(aClient *cptr, aChannel *chptr)
{
	chanMember **curr, *tmp;

	for (curr = &chptr->members; (tmp = *curr); curr = &tmp->nextuser) {
		if (tmp->cptr == cptr) {
			*curr = tmp->nextuser;
			break;
		}
	}
	for (curr = &cptr->user->channel; (tmp = *curr); curr = &tmp->nextchan) {
		if (tmp->chptr == chptr) {
			*curr = tmp->nextchan;
			free_chanmember(tmp);
			break;
		}
	}
	cptr->user->joined--;
	sub1_from_channel(chptr);
	return *curr;
}

int can_send(aClient *cptr, aChannel *chptr, char *msg)
{
	chanMember *cm;

	if (IsServer(cptr) || IsULine(cptr)) {
		return 0;
	}
	if ((cm = find_user_member(cptr->user->channel, chptr)) == NULL) {
		if (chptr->mode.mode & MODE_MODERATED) {
			return MODE_MODERATED;
		}
		if (chptr->mode.mode & MODE_NOPRIVMSGS) {
			return MODE_NOPRIVMSGS;
		}
		if (chptr->mode.mode & MODE_NOCOLOUR && msg != NULL && msg_has_colour(msg) == 1) {
			return ERR_NOCOLOURSONCHAN;
		}
		if (MyClient(cptr) && is_banned(cptr, chptr)) {
			return MODE_BAN;
		}
	}
	else {
		if (chptr->mode.mode & MODE_MODERATED && !(cm->flags & (CHFL_CHANADMIN|CHFL_CHANOP|CHFL_HALFOP
		  |CHFL_VOICE))) {
			return MODE_MODERATED;
		}
		if (chptr->mode.mode & MODE_NOCOLOUR && msg_has_colour(msg) == 1) {
			return ERR_NOCOLOURSONCHAN;
		}
		if (cm->bans > 0 && !(cm->flags & (CHFL_CHANADMIN|CHFL_CHANOP|CHFL_HALFOP|CHFL_VOICE))) {
			return MODE_BAN;
		}
	}
	return 0;
}

int check_channelmask(aClient *sptr, aClient *cptr, char *chname)
{
	char *s;

	if (*chname == '&' && IsServer(cptr)) {
		return -1;
	}
	if ((s = rindex(chname, ':')) == NULL) {
		return 0;
	}
	s++;
	if (match(s, me.name) || (IsServer(cptr) && match(s, cptr->name))) {
		if (MyClient(sptr)) {
			sendto_one(sptr, err_str(ERR_BADCHANMASK), me.name, sptr->name, chname);
		}
		return -1;
	}
	return 0;
}

int can_join(aClient *sptr, aChannel *chptr, char *key)
{
	Link *lp;

	if (sptr == NULL || sptr->user == NULL || IsULine(sptr)) {
		return 0;
	}
	for (lp = sptr->user->invited; lp != NULL; lp = lp->next) {
		if (lp->value.chptr == chptr) {
			return 0;
		}
	}
	if (has_invex(sptr, chptr)) {
		return 0;
	}
	if (is_banned(sptr, chptr)) {
		return ERR_BANNEDFROMCHAN;
	}
	if (chptr->mode.mode & MODE_INVITEONLY) {
		return ERR_INVITEONLYCHAN;
	}
	if (chptr->mode.mode & MODE_REGONLY && !HasMode(sptr, UMODE_REGNICK)) {
		return ERR_NEEDREGGEDNICK;
	}
	if (chptr->mode.mode & MODE_OPERONLY && !HasMode(sptr, UMODE_OPER)) {
		return ERR_NOPRIVILEGES;
	}
	if (chptr->mode.mode & MODE_ADMINONLY && !HasMode(sptr, UMODE_NETADMIN|UMODE_ADMIN|UMODE_COADMIN)) {
		return ERR_NOPRIVILEGES;
	}
	if (*chptr->mode.key && (BadPtr(key) || irccmp(chptr->mode.key, key))) {
		return ERR_BADCHANNELKEY;
	}
	if (chptr->mode.limit && chptr->users >= chptr->mode.limit) {
		return ERR_BADCHANNELKEY;
	}
	return 0;
}

void clean_channelname(unsigned char *cn)
{
	while (*cn++) {
		if (*cn < 33 || *cn == ',' || *cn == 160) {
			*cn = '\0';
			return;
		}
	}
	return;
}

int check_channelname(aClient *cptr, unsigned char *cn)
{
	if (!MyClient(cptr)) {
		return 1;
	}
	while (*cn) {
		if (*cn < 33 || *cn == ',' || *cn == 160) {
			sendto_one(cptr, err_str(ERR_BADCHANNAME), me.name, cptr->name, cn);
			return 0;
		}
		cn++;
	}
	return 1;
}

aChannel *get_channel(aClient *cptr, char *chname, int flag)
{
	aChannel *chptr = NULL;
	int len;

	if (BadPtr(chname)) {
		return NULL;
	}

	len = strlen(chname);
	if (MyClient(cptr) && len > CHANNELLEN) {
		chname[CHANNELLEN] = '\0';
		len = CHANNELLEN;
	}
	if ((chptr = find_channel(chname, NULL)) == NULL && flag == CREATE) {
		chptr = make_channel();
		strncpyzt(chptr->chname, chname, len + 1);
		if (channel != NULL) {
			channel->prevch = chptr;
		}
		chptr->prevch = NULL;
		chptr->nextch = channel;
		channel = chptr;
		chptr->channelts = timeofday;
		add_to_channel_hash_table(chname, chptr);
		Count.chan++;
	}
	return chptr;
}

static int list_length(Link *lp)
{
	int count;
	for (count = 0; lp != NULL; lp = lp->next) {
		count++;
	}
	return count;
}

void add_invite(aClient *cptr, aChannel *chptr)
{
	Link *inv, **tmp;

	if (cptr == NULL || cptr->user == NULL) {
		ircdlog(LOG_ERROR, "received NULL client/user poiunter in add_invite [cptr %lX "
			"user %lX]", cptr, cptr->user);
		return;
	}

	del_invite(cptr, chptr);
	if (list_length(cptr->user->invited) >= MAXCHANSPERUSER) {
		del_invite(cptr, cptr->user->invited->value.chptr);
	}

	inv = make_slink();
	inv->value.cptr = cptr;
	inv->next = chptr->invites;
	chptr->invites = inv;

	for (tmp = &(cptr->user->invited); *tmp != NULL; tmp = &((*tmp)->next));
	inv = make_slink();
	inv->value.chptr = chptr;
	inv->next = NULL;
	*tmp = inv;
	return;
}

void del_invite(aClient *cptr, aChannel *chptr)
{
	Link **inv, *tmp;
	for (inv = &(chptr->invites); (tmp = *inv); inv = &tmp->next) {
		if (tmp->value.cptr == cptr) {
			*inv = tmp->next;
			free_slink(tmp);
			break;
		}
	}
	for (inv = &(cptr->user->invited); (tmp = *inv); inv = &tmp->next) {
		if (tmp->value.chptr == chptr) {
			*inv = tmp->next;
			free_slink(tmp);
			break;
		}
	}
	return;
}

void send_topic_burst(aClient *cptr)
{
	aChannel *chptr;
	aClient *acptr;

	for (chptr = channel; chptr != NULL; chptr = chptr->nextch) {
		if (*chptr->topic != '\0') {
			sendto_one(cptr, ":%s TOPIC %s %s %ld :%s", me.name, chptr->chname,
				chptr->topic_nick, chptr->topic_time, chptr->topic);
		}
	}
	for (acptr = client; acptr != NULL; acptr = acptr->next) {
		if (!IsPerson(acptr) || acptr->from == cptr) {
			continue;
		}
		if (acptr->user->away != NULL) {
			sendto_one(cptr, ":%s AWAY :%s", acptr->name, acptr->user->away);
		}
	}
	return;
}

void send_names(aClient *sptr, aChannel *chptr)
{
	int spos, flag = 1, idx = 0, mlen = strlen(me.name) + NICKLEN + 7;
	chanMember *cm;
	aClient *acptr;
	char *s;

	if (PubChannel(chptr)) {
		buf[idx++] = '=';
	}
	else if (SecretChannel(chptr)) {
		buf[idx++] = '@';
	}
	else {
		buf[idx++] = '*';
	}
	buf[idx++] = ' ';
	for (s = chptr->chname; *s; s++) {
		buf[idx++] = *s;
	}
	buf[idx++] = ' ';
	buf[idx++] = ':';
	buf[idx] = '\0';
	spos = idx;

	for (cm = chptr->members; cm != NULL; cm = cm->nextuser) {
		acptr = cm->cptr;
		if (HasMode(acptr, UMODE_INVISIBLE) && !IsMember(sptr, chptr)) {
			continue;
		}
		if (cm->flags & CHFL_CHANOP) {
			buf[idx++] = '@';
		}
		else if (cm->flags & CHFL_HALFOP) {
			buf[idx++] = '%';
		}
		else if (cm->flags & CHFL_VOICE) {
			buf[idx++] = '+';
		}
		for (s = acptr->name; *s; s++) {
			buf[idx++] = *s;
		}
		buf[idx++] = ' ';
		buf[idx] = '\0';
		flag = 1;

		if (mlen + idx + NICKLEN > BUFSIZE - 3) {
			sendto_one(sptr, rpl_str(RPL_NAMREPLY), me.name, sptr->name, buf);
			idx = spos;
			flag = 0;
		}
	}
	if (flag == 1) {
		sendto_one(sptr, rpl_str(RPL_NAMREPLY), me.name, sptr->name, buf);
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFNAMES), me.name, sptr->name, chptr->chname);
	return;
}

void send_list(aClient *sptr, int numsend)
{
	char modestuff[TOPICLEN + 3 + KEYLEN + 8 + 3 + 1], tmpbuf[3 + KEYLEN + 8 + 3 + 1];
	LOpts *lopt = sptr->user->lopt;
	aChannel *chptr;
	int hashnum;

	for (hashnum = lopt->starthash; hashnum < CH_MAX; hashnum++) {
		if (numsend > 0) {
			for (chptr = (aChannel *)hash_get_chan_bucket(hashnum); chptr != NULL;
			  chptr = chptr->hnextch) {
				if (SecretChannel(chptr) && !IsMember(sptr, chptr) && !HasMode(sptr, UMODE_SADMIN)) {
					continue;
				}
				if ((!lopt->showall) && ((chptr->users < lopt->usermin) || ((lopt->usermax >= 0) &&
				  (chptr->users > lopt->usermax)) || ((chptr->channelts||1) < lopt->chantimemin) ||
				  (chptr->topic_time < lopt->topictimemin) || (chptr->channelts > lopt->chantimemax) ||
				  (chptr->topic_time > lopt->topictimemax) || (lopt->nolist &&
				  find_str_link(lopt->nolist, chptr->chname)) || (lopt->yeslist && 
				  !find_str_link(lopt->yeslist, chptr->chname)))) {
					continue;
				}
				if (HasMode(sptr, UMODE_SADMIN)) {
					*modebuf = *parabuf = '\0';
					channel_modes(sptr, modebuf, parabuf, chptr);
					ircsprintf(tmpbuf, " [%s%s%s]", modebuf,
						*parabuf ? " " : "",
						*parabuf ? parabuf : "");
					ircsprintf(modestuff, "%-2s %s", tmpbuf, chptr->topic);
					sendto_one(sptr, rpl_str(RPL_LIST), me.name, sptr->name,
						chptr->chname, chptr->users, modestuff);
				}
				else if (ShowChannel(sptr, chptr)) {
					sendto_one(sptr, rpl_str(RPL_LIST), me.name, sptr->name,
						chptr->chname, chptr->users, chptr->topic);
				}
				numsend--;
			}
		}
		else {
			break;
		}
	}
	if (hashnum == CH_MAX) {
		Link *lp, *next;

		sendto_one(sptr, rpl_str(RPL_LISTEND), me.name, sptr->name);
		for (lp = lopt->yeslist; lp != NULL; lp = next) {
			next = lp->next;
			MyFree(lp->value.cp);
			free_slink(lp);
		}
		for (lp = lopt->nolist; lp != NULL; lp = next) {
			next = lp->next;
			MyFree(lp->value.cp);
			free_slink(lp);
		}
		MyFree(sptr->user->lopt);
		sptr->user->lopt = NULL;
		return;
	}
	lopt->starthash = hashnum;
	return ;
}
