/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/drone.c: drone-bot detection and prevension
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: drone.c,v 1.5 2002/11/23 21:51:06 rageircd Exp $
 */

#include "conf2.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "patchlevel.h"
#include "config.h"
#include "memory.h"
#include "send.h"
#include "h.h"
#include <time.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int spam_time = MIN_JOIN_LEAVE_TIME;
int spam_num = MAX_JOIN_LEAVE_COUNT;

#ifdef HAVE_THROTTLE_ON
extern int throttle_force(char *);
#endif

static int detect_acebot(char *username, char *gcos)
{
	unsigned char *x;

	if (*username == '~') {
		username++;
	}
	if (strlen(username) <= 2) {
		return 0;
	}
	for (x = (unsigned char *)username; *x != '\0'; x++) {
		if (*x < 'A' || *x > 'Z') {
			return 0;
		}
	}
	if (irccmp(username, gcos)) {
		return 0;
	}
	return 1;
}

int drone_check_spambot(aClient *sptr, char spamchar)
{
	if (spamchar == 'x') {
		ircstp->is_ref++;
#ifdef HAVE_THROTTLE_ON
		if (THROTTLE_REJECTED_CLIENTS == 1) {
			throttle_force(sptr->host);
		}
#endif
		sendto_realops_lev(REJ_LEV, "Rejecting possible spambot: %s (single char user-"
			"given userhost: %c)", get_client_name(sptr, FALSE), spamchar);
		return 1;
	}
	return 0;
}

int drone_check_acebot(aClient *sptr)
{
	if (detect_acebot(sptr->username, sptr->info) == 1) {
		ircstp->is_ref++;
#ifdef HAVE_THROTTLE_ON
		if (THROTTLE_REJECTED_CLIENTS == 1) {
			throttle_force(sptr->host);
		}
#endif
		sendto_realops_lev(REJ_LEV, "Rejecting possible acebot-style drone: %s (%s@%s) [%s]",
			sptr->name, sptr->username, sptr->host, sptr->info);
		return 1;
	}
	return 0;
}
