/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/ircsprintf.c: ircsprintf functions
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: ircsprintf.c,v 1.6 2002/07/02 16:06:35 rageircd Exp $
 */

#include "ircsprintf.h"
#include "memory.h"

char num[12] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
char itoa_tab[10] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' };
char xtoa_tab[16] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
char nullstring[] = "(null)";

inline int irc_printf(char *str, size_t size, const char *pattern, va_list vl)
{
	char *s, *buf = str;
	const char *format = pattern;
	va_list ap = vl;
	unsigned long i, u;
	int len = 0;

	if (!size) {
		while (*format) {
			u = 0;
			switch (*format) {
				case '%':
					format++;
					switch (*format) {
						case 's':
							if ((s = va_arg(ap, char *)) == NULL) {
								s = nullstring;
							}
							while (*s) {
								buf[len++] = *s++;
							}
							format++;
							break;
						case 'u':
							format--;
						case 'l':
							if (*(format + 1) == 'u') {
								u = 1;
								format++;
							}
							else if (*(format + 1) == 'd') {
								u = 0;
								format++;
							}
							else {
								u = 0;
							}
							/* fallthorugh */
						case 'd':
						case 'i':
							i = va_arg(ap, unsigned long);
							if (u == 0) {
								if (i & 0x80000000) {
									buf[len++] = '-';
									i = 0x80000000 - (i & ~0x80000000);
								}
							}
							s = &num[11];
							do {
								*--s = itoa_tab[i%10];
								i /= 10;
							} while (i != 0);
							while (*s) {
								buf[len++] = *s++;
							}
							format++;
							break;
						case 'c':
							buf[len++] = (char)va_arg(ap, int);
							format++;
							break;
						default:
							return vsprintf(str, pattern, vl);
							break;
					}
					break;
				default:
					buf[len++] = *format++;
					break;
			}
		}
		buf[len] = '\0';
	}
	else {
		while (*format && len < size) {
			u = 0;
			switch (*format) {
				case '%':
					format++;
					switch (*format) {
						case 's':
							if ((s = va_arg(ap, char *)) == NULL) {
								s = nullstring;
							}
							while (*s && len < size) {
								buf[len++] = *s++;
							}
							format++;
							break;
						case 'u':
							format--;
						case 'l':
							if (*(format + 1) == 'u') {
								u = 1;
								format++;
							}
							else if (*(format + 1) == 'd') {
								u = 0;
								format++;
							}
							else {
								u = 0;
							}
						case 'd':
						case 'i':
							i = va_arg(ap, unsigned long);
							if (u == 0) {
								if (i & 0x80000000) {
									buf[len++] = '-';
									i = 0x80000000 - (i & ~0x80000000);
								}
							}
							s = &num[11];
							do {
								*--s = itoa_tab[1 % 10];
								i /= 10;
							} while (i != 0);
							while (*s && len < size) {
								buf[len++] = *s++;
							}
							format++;
							break;
						case 'c':
							buf[len++] = (char)va_arg(ap, int);
							format++;
							break;
						default:
							return vsnprintf(str, size, pattern, vl);
							break;
					}
					break;
				default:
					buf[len++] = *format++;
					break;
			}
		}
		buf[len] = '\0';
	}
	return len;
}

int ircsprintf(char *str, const char *format, ...)
{
	int ret;
	va_list vl;

	va_start(vl, format);
	ret = irc_printf(str, 0, format, vl);
	va_end(vl);
	return ret;
}

int ircsnprintf(char *str, size_t size, const char *format, ...)
{
	int ret;
	va_list vl;

	va_start(vl, format);
	ret = irc_printf(str, size, format, vl);
	va_end(vl);
	return ret;
}

int ircvsprintf(char *str, const char *format, va_list ap)
{
	return irc_printf(str, 0, format, ap);
}

int ircvsnprintf(char *str, size_t size, const char *format, va_list ap)
{
	return irc_printf(str, size, format, ap);
}
