/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/parse.c: internal parsing functions
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: parse.c,v 1.85 2002/12/15 21:26:20 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "msg.h"
#include "sys.h"
#include "numeric.h"
#include "h.h"
#include "memory.h"

#ifdef HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif

aCommand *CommandHash[256];
static char *para[MAXPARA + 1];
static char sender[HOSTLEN + 1];

void init_CommandHash(void)
{
	memset(CommandHash, '\0', sizeof(CommandHash));
#ifdef STATIC_MODULES
	addCommand("CHANSERV",		m_chanserv,	1,		MSG_NONE);
	addCommand("CS",		m_chanserv,	1,		MSG_NONE);
	addCommand("NICKSERV",		m_nickserv,	1,		MSG_NONE);
	addCommand("NS",		m_nickserv,	1,		MSG_NONE);
	addCommand("MEMOSERV",		m_memoserv,	1,		MSG_NONE);
	addCommand("MS",		m_memoserv,	1,		MSG_NONE);
	addCommand("OPERSERV",		m_operserv,	1,		MSG_NONE);
	addCommand("OS",		m_operserv,	1,		MSG_NONE);
	addCommand("STATSERV",		m_statserv,	1,		MSG_NONE);
	addCommand("SS",		m_statserv,	1,		MSG_NONE);
	addCommand("SERVICES",		m_services,	1,		MSG_NONE);
	addCommand("IDENTIFY",		m_identify,	1,		MSG_NONE);
	addCommand("AWAY",		m_away,		MAXPARA,	MSG_RESETIDLE);
	addCommand("MAP",		m_map,		MAXPARA,	MSG_NONE);
	addCommand("MKPASSWD",		m_mkpasswd,	MAXPARA,	MSG_NONE);
	addCommand("OPER",		m_oper,		MAXPARA,	MSG_NONE);
	addCommand("VERSION",		m_version,	MAXPARA,	MSG_UNREGUSE);
	addCommand("SVSMODE",		m_svsmode,	MAXPARA,	MSG_NONE);
	addCommand("SVSNICK",		m_svsnick,	MAXPARA,	MSG_NONE);
	addCommand("JOIN",		m_join,		MAXPARA,	MSG_RESETIDLE);
	addCommand("PART",		m_part,		MAXPARA,	MSG_RESETIDLE);
	addCommand("KLINE",		m_kline,	MAXPARA,	MSG_NONE);
	addCommand("UNKLINE",		m_unkline,	MAXPARA,	MSG_NONE);
	addCommand("DIE",		m_die,		MAXPARA,	MSG_NONE);
	addCommand("RESTART",		m_restart,	MAXPARA,	MSG_NONE);
	addCommand("ZLINE",		m_zline,	MAXPARA,	MSG_NONE);
	addCommand("SQLINE",		m_sqline,	MAXPARA,	MSG_NONE);
	addCommand("UNSQLINE",		m_unsqline,	MAXPARA,	MSG_NONE);
	addCommand("WHOIS",		m_whois,	MAXPARA,	MSG_NONE);
	addCommand("WHOWAS",		m_whowas,	MAXPARA,	MSG_NONE);
	addCommand("ADMIN",		m_admin,	MAXPARA,	MSG_UNREGUSE);
	addCommand("INFO",		m_info,		MAXPARA,	MSG_NONE);
	addCommand("DALINFO",		m_dalinfo,	MAXPARA,	MSG_NONE);
	addCommand("LINKS",		m_links,	MAXPARA,	MSG_NONE);
	addCommand("KICK",		m_kick,		MAXPARA,	MSG_NONE);
	addCommand("VHOST",		m_vhost,	MAXPARA,	MSG_NONE);
	addCommand("MODE",		m_mode,		MAXPARA,	MSG_NONE);
	addCommand("TOPIC",		m_topic,	MAXPARA,	MSG_NONE);
	addCommand("INVITE",		m_invite,	MAXPARA,	MSG_NONE);
	addCommand("NAMES",		m_names,	MAXPARA,	MSG_NONE);
	addCommand("TIME",		m_time,		MAXPARA,	MSG_NONE);
	addCommand("PING",		m_ping,		MAXPARA,	MSG_NONE);
	addCommand("PONG",		m_pong,		MAXPARA,	MSG_NONE);
	addCommand("QUIT",		m_quit,		MAXPARA,	MSG_NONE);
	addCommand("PASS",		m_pass,		MAXPARA,	MSG_UNREGUSE);
	addCommand("USERHOST",		m_userhost,	MAXPARA,	MSG_NONE);
	addCommand("ISON",		m_ison,		MAXPARA,	MSG_NONE);
	addCommand("MOTD",		m_motd,		MAXPARA,	MSG_NONE);
	addCommand("RULES",		m_rules,	MAXPARA,	MSG_NONE);
	addCommand("OPERMOTD",		m_opermotd,	MAXPARA,	MSG_NONE);
	addCommand("SVSNOOP",		m_svsnoop,	MAXPARA,	MSG_NONE);
	addCommand("LIST",		m_list,		MAXPARA,	MSG_NONE);
	addCommand("SAMODE",		m_samode,	MAXPARA,	MSG_NONE);
	addCommand("CAPAB",		m_capab,	MAXPARA,	MSG_UNREGUSE);
	addCommand("SJOIN",		m_sjoin,	MAXPARA,	MSG_NONE);
	addCommand("REHASH",		m_rehash,	MAXPARA,	MSG_NONE);
	addCommand("HTM",		m_htm,		MAXPARA,	MSG_NONE);
	addCommand("BURST",		m_burst,	MAXPARA,	MSG_NONE);
	addCommand("SVINFO",		m_svinfo,	MAXPARA,	MSG_UNREGUSE);
	addCommand("USERS",		m_users,	MAXPARA,	MSG_NONE);
	addCommand("ERROR",		m_error,	MAXPARA,	MSG_UNREGUSE);
	addCommand("LUSERS",		m_lusers,	MAXPARA,	MSG_NONE);
	addCommand("CONNECT",		m_connect,	MAXPARA,	MSG_NONE);
	addCommand("WALLOPS",		m_wallops,	MAXPARA,	MSG_NONE);
	addCommand("LOCOPS",		m_locops,	MAXPARA,	MSG_NONE);
	addCommand("GLOBOPS",		m_globops,	MAXPARA,	MSG_NONE);
	addCommand("GNOTICE",		m_gnotice,	MAXPARA,	MSG_NONE);
	addCommand("SQUIT",		m_squit,	MAXPARA,	MSG_NONE);
	addCommand("CHATOPS",		m_chatops,	MAXPARA,	MSG_NONE);
	addCommand("CLOSE",		m_close,	MAXPARA,	MSG_NONE);
	addCommand("NICK",		m_nick,		MAXPARA,	MSG_UNREGUSE);
	addCommand("USER",		m_user,		MAXPARA,	MSG_UNREGUSE);
	addCommand("PRIVMSG",		m_private,	MAXPARA,	MSG_RESETIDLE);
	addCommand("NOTICE",		m_notice,	MAXPARA,	MSG_NONE);
	addCommand("KILL",		m_kill,		MAXPARA,	MSG_NONE);
	addCommand("SVSKILL",		m_svskill,	MAXPARA,	MSG_NONE);
	addCommand("AKILL",		m_akill,	MAXPARA,	MSG_NONE);
	addCommand("RAKILL",		m_rakill,	MAXPARA,	MSG_NONE);
	addCommand("DKEY",		m_dkey,		MAXPARA,	MSG_UNREGUSE);
	addCommand("TRACE",		m_trace,	MAXPARA,	MSG_NONE);
	addCommand("WATCH",		m_watch,	MAXPARA,	MSG_NONE);
	addCommand("DCCALLOW",		m_dccallow,	MAXPARA,	MSG_NONE);
	addCommand("SILENCE",		m_silence,	MAXPARA,	MSG_NONE);
	addCommand("STATS",		m_stats,	MAXPARA,	MSG_NONE);
	addCommand("WHO",		m_who,		MAXPARA,	MSG_NONE);
	addCommand("SERVER",		m_server,	MAXPARA,	MSG_UNREGUSE);
	addCommand("KNOCK",		m_knock,	MAXPARA,	MSG_NONE);
	addCommand("SET",		m_set,		MAXPARA,	MSG_NONE);
	addCommand("SGLINE",		m_sgline,	MAXPARA,	MSG_NONE);
	addCommand("UNSGLINE",		m_unsgline,	MAXPARA,	MSG_NONE);
	addCommand("ACCEPT",		m_accept,	MAXPARA,	MSG_NONE);
	addCommand("SAJOIN",		m_sajoin,	MAXPARA,	MSG_NONE);
	addCommand("SAKICK",		m_sakick,	MAXPARA,	MSG_NONE);
	addCommand("SZLINE",		m_szline,	MAXPARA,	MSG_NONE);
	addCommand("UNSZLINE",		m_unszline,	MAXPARA,	MSG_NONE);
	addCommand("SNICK",		m_snick,	MAXPARA,	MSG_NONE);
	addHook(HOOKTYPE_JOIN, m_join, NULL);
#endif
	return;
}

static inline aCommand *findCommand(char *cmd, int (*func)())
{
	aCommand *p;
	for (p = CommandHash[touppertab[(unsigned char)*cmd]]; p != NULL; p = p->next) {
		if (!mycmp(p->cmd, cmd)) {
			if (func != NULL && (p->func != func)) {
				continue;
			}
			return p;
		}
	}
	return NULL;
}

void addCommand(char *cmd, int (*func)(), int paras, int flags)
{
	aCommand *newcmd = NULL;
	if ((newcmd = findCommand(cmd, NULL)) == NULL) {
		newcmd = (aCommand *)MyMalloc(sizeof(aCommand));
		DupString(newcmd->cmd, cmd);
		newcmd->func = func;
		newcmd->count = 0;
		newcmd->paras = paras;
		newcmd->unreg_use = (flags & MSG_UNREGUSE) ? 1 : 0;
		newcmd->reset_idle = (flags & MSG_RESETIDLE) ? 1 : 0;
		newcmd->bytes = 0L;
		newcmd->prev = NULL;
		if ((newcmd->next = CommandHash[touppertab[(unsigned char)*cmd]]) != NULL) {
			newcmd->next->prev = newcmd;
		}
		CommandHash[touppertab[(unsigned char)*cmd]] = newcmd;
	}
	return;
}

void delCommand(char *cmd, int (*func)())
{
	aCommand *p = NULL;
	if ((p = findCommand(cmd, func)) != NULL) {
		if (p->prev != NULL) {
			p->prev->next = p->next;
		}
		else {
			CommandHash[touppertab[(unsigned char)*cmd]] = p->next;
		}
		if (p->next != NULL) {
			p->next->prev = p->prev;
		}
		MyFree(p->cmd);
		MyFree(p);
	}
	return;
}

static int
cancel_clients(aClient *cptr,
	       aClient *sptr,
	       char *cmd)
{
   /*
    * kill all possible points that are causing confusion here, I'm not
    * sure I've got this all right... - avalon
    * 
    * knowing avalon, probably not.
    */
   /*
    * * with TS, fake prefixes are a common thing, during the * connect
    * burst when there's a nick collision, and they * must be ignored
    * rather than killed because one of the * two is surviving.. so we
    * don't bother sending them to * all ops everytime, as this could
    * send 'private' stuff * from lagged clients. we do send the ones
    * that cause * servers to be dropped though, as well as the ones
    * from * non-TS servers -orabidoo
    */
   /*
    * Incorrect prefix for a server from some connection.  If it is a
    * client trying to be annoying, just QUIT them, if it is a server
    * then the same deal.
    */
   if (IsServer(sptr) || IsMe(sptr)) {
		/* Sorry, but at the moment this is just too much for even opers 
		 to see. -Rak */
		/* or we could just take out the message. <EG>  -wd */
		 sendto_realops_lev(DEBUG_LEV, "Message for %s[%s] from %s",
								  sptr->name, sptr->from->name,
								  get_client_name(cptr, (IsServer(cptr) ? HIDEME : TRUE)));
		if (IsServer(cptr)) {
			sendto_realops_lev(DEBUG_LEV,
									 "Not dropping server %s (%s) for Fake Direction",
									 cptr->name, sptr->name);
			return -1;
		}
		
		if (IsClient(cptr))
		  sendto_realops_lev(DEBUG_LEV,
									"Would have dropped client %s (%s@%s) [%s from %s]",
									cptr->name, cptr->username, cptr->host,
									cptr->user->server, cptr->from->name);
		return -1;
		/*
		 * return exit_client(cptr, cptr, &me, "Fake Direction");
		 */
	}
	/*
	 * Ok, someone is trying to impose as a client and things are
	 * confused.  If we got the wrong prefix from a server, send out a
	 * kill, else just exit the lame client.
	 */
	if (IsServer(cptr)) {
	  /*
		* * If the fake prefix is coming from a TS server, discard it *
		* silently -orabidoo
		* also drop it if we're gonna kill services by not doing so }:/
		*/
		if (DoesTS(cptr)) {
			if (sptr->user)
			  sendto_realops_lev(DEBUG_LEV,
										"Message for %s[%s@%s!%s] from %s (TS, ignored)",
										sptr->name, sptr->username, sptr->host,
										sptr->from->name, get_client_name(cptr, TRUE));
			return 0;
		}
		else {
			if (sptr->user)
			  sendto_realops_lev(DEBUG_LEV,
										"Message for %s[%s@%s!%s] from %s",
										sptr->name, sptr->username, sptr->host,
										sptr->from->name,
										get_client_name(cptr, (IsServer(cptr) ? HIDEME : TRUE)));
			if(IsULine(sptr)) {
				sendto_realops_lev(DEBUG_LEV,
										 "Would have killed U:lined client %s for fake direction",
										 sptr->name);
				return 0;
			}
			sendto_serv_butone(NULL,
									 ":%s KILL %s :%s (%s[%s] != %s, Fake Prefix)",
									 me.name, sptr->name, me.name,
									 sptr->name, sptr->from->name,
									 get_client_name(cptr, (IsServer(cptr) ? HIDEME : TRUE)));
			sptr->flags |= FLAGS_KILLED;
			return exit_client(cptr, sptr, &me, "Fake Prefix");
		}
	}
	return exit_client(cptr, cptr, &me, "Fake prefix");
}

static void
remove_unknown(aClient *cptr,
	       char *sender,
	       char *buffer)
{
   if (!IsRegistered(cptr))
      return;

   if (IsClient(cptr)) {
      sendto_realops_lev(DEBUG_LEV,
	 "Weirdness: Unknown client prefix (%s) from %s, Ignoring %s",
			 buffer,
			 get_client_name(cptr, FALSE), sender);
      return;
   }
   /*
    * Not from a server so don't need to worry about it.
    */
   if (!IsServer(cptr))
      return;
   /*
    * Do kill if it came from a server because it means there is a
    * ghost user on the other server which needs to be removed. -avalon
    * Tell opers about this. -Taner
    */
   if (!strchr(sender, '.'))
      sendto_one(cptr, ":%s KILL %s :%s (%s(?) <- %s)",
		 me.name, sender, me.name, sender,
		 get_client_name(cptr, FALSE));
   else {
      sendto_realops_lev(DEBUG_LEV, "Unknown prefix (%s) from %s, Squitting %s", buffer, get_client_name(cptr, HIDEME), sender);
      sendto_one(cptr, ":%s SQUIT %s :(Unknown prefix (%s) from %s)",
	       me.name, sender, buffer, get_client_name(cptr, HIDEME));
   }
}

/*
 * parse a buffer.
 * 
 * NOTE: parse() should not be called recusively by any other functions!
 */
int parse(aClient *cptr, char *buffer, char *bufend)
{
   aClient *from = cptr;
   char *ch, *s;
   int i, numeric = 0, paramcount;
   aCommand *cmptr = NULL;

   Debug((DEBUG_DEBUG, "Parsing %s: %s", get_client_name(cptr, TRUE), buffer));

   if (IsDead(cptr))
      return -1;

   s = sender;
   *s = '\0';

   for (ch = buffer; *ch == ' '; ch++);	/* skip spaces */

   para[0] = from->name;
   if (*ch == ':') 
   {
      /*
       * Copy the prefix to 'sender' assuming it terminates with
       * SPACE (or NULL, which is an error, though).
       */

      for (++ch; *ch && *ch != ' '; ++ch)
	 if (s < (sender + HOSTLEN))
	    *s++ = *ch;		
      *s = '\0';

      /*
       * Actually, only messages coming from servers can have the
       * prefix--prefix silently ignored, if coming from a user
       * client...
       * 
       * ...sigh, the current release "v2.2PL1" generates also null
       * prefixes, at least to NOTIFY messages (e.g. it puts
       * "sptr->nickname" as prefix from server structures where it's
       * null--the following will handle this case as "no prefix" at
       * all --msa  (": NOTICE nick ...")
       */

      if (*sender && IsServer(cptr)) 
      {
         from = find_client(sender, (aClient *) NULL);

         /*
          * okay, this doesn't seem to do much here.
          * from->name _MUST_ be equal to sender. That's what find_client does.
          * find_client will find servers too, and since we don't use server
          * masking, the find server call is useless (and very wasteful).
          * now, there HAS to be a from and from->name and sender have to be the same
          * for us to get to the next if. but the next if starts out with if(!from)
          * so this is UNREACHABLE CODE! AGH! - lucas
          *
          *  if (!from || mycmp(from->name, sender))
          *     from = find_server(sender, (aClient *) NULL);
          *  else if (!from && strchr(sender, '@'))
	  *     from = find_nickserv(sender, (aClient *) NULL);
          */

	 para[0] = sender;
	 /*
	  * Hmm! If the client corresponding to the prefix is not
	  * found--what is the correct action??? Now, I will ignore the
	  * message (old IRC just let it through as if the prefix just
	  * wasn't there...) --msa
	  */
	 if (!from) 
         {
            Debug((DEBUG_ERROR, "Unknown prefix (%s)(%s) from (%s)",
                   sender, buffer, cptr->name));

            ircstp->is_unpf++;
            remove_unknown(cptr, sender, buffer);
		 
            return -1;
	 }

	 if (from->from != cptr) 
         {
	    ircstp->is_wrdi++;
	    Debug((DEBUG_ERROR, "Message (%s) coming from (%s)",
		   buffer, cptr->name));

	    return cancel_clients(cptr, from, buffer);
	 }
      }
      while (*ch == ' ')
	 ch++;
   }

   if (*ch == '\0') 
   {
      ircstp->is_empt++;
      Debug((DEBUG_NOTICE, "Empty message from host %s:%s",
	     cptr->name, from->name));
      return (-1);
   }

   /*
    * Extract the command code from the packet.  Point s to the end
    * of the command code and calculate the length using pointer
    * arithmetic.  Note: only need length for numerics and *all*
    * numerics must have parameters and thus a space after the command
    * code. -avalon
    * 
    * ummm???? - Dianora
    */

   /* check for numeric */
   if (*(ch + 3) == ' ' && isdigit(*ch) && isdigit(*(ch + 1)) && isdigit(*(ch + 2))) 
   {
      cmptr = (aCommand *)NULL;
      numeric = (*ch - '0') * 100 + (*(ch + 1) - '0') * 10 + (*(ch + 2) - '0');
      paramcount = MAXPARA;
      ircstp->is_num++;
      s = ch + 3;
      *s++ = '\0';
   }
   else 
   {


      s = strchr(ch, ' ');

      if (s)
	 *s++ = '\0';

      if ((cmptr = findCommand(ch, NULL)) == NULL || cmptr->cmd == NULL)
      {
	 /*
          * only send error messages to things that actually sent buffers to us
          * and only people, too.
	  */
	 if (buffer[0] != '\0') 
         {
	    if (IsPerson(from))
	       sendto_one(from, ":%s %d %s %s :Unknown command",
			  me.name, ERR_UNKNOWNCOMMAND, from->name, ch);
	    Debug((DEBUG_ERROR, "Unknown (%s) from %s",
		   ch, get_client_name(cptr, TRUE)));
	 }
	 ircstp->is_unco++;
	 return -1;
      }

      paramcount = cmptr->paras;
      i = bufend - ((s) ? s : ch);
      cmptr->bytes += i;
      /*
       * Allow only 1 msg per 2 seconds (on average) to prevent
       * dumping. to keep the response rate up, bursts of up to 5 msgs
       * are allowed -SRB Opers can send 1 msg per second, burst of ~20
       * -Taner
       */
	/* cmptr->flags variable removed, so dont bother referencing it...
	 * it was always 1 on every command, anyway.
	 */
      if (!IsServer(cptr)) {
	if (NOOPERFLOOD == 1 && HasMode(cptr, UMODE_OPER)) {
		cptr->since += (cptr->receiveM % 10) ? 1 : 0;
	}
	else {
		cptr->since += (2 + i / 120);
	}
      }
   }
   /*
    * Must the following loop really be so devious? On surface it
    * splits the message to parameters from blank spaces. But, if
    * paramcount has been reached, the rest of the message goes into
    * this last parameter (about same effect as ":" has...) --msa
    */

   /*
    * Note initially true: s==NULL || *(s-1) == '\0' !! 
    */

   i = 1;
   if (s) 
   {
      if (paramcount > MAXPARA)
	 paramcount = MAXPARA;
      for (;;) 
      {
	 while (*s == ' ')
	    *s++ = '\0';

	 if (*s == '\0')
	    break;
	 if (*s == ':') 
         {
	    /* The rest is a single parameter */
	    para[i++] = s + 1;
	    break;
	 }
	 para[i++] = s;
	 if (i >= paramcount)
	    break;

         while(*s && *s != ' ')
            s++;
      }
   }

   para[i] = NULL;
   if (cmptr == (aCommand *) NULL)
      return (do_numeric(numeric, cptr, from, i, para));

   cmptr->count++;

   /*
    * patch to avoid server flooding from unregistered connects 
    */
   /*
    * check allow_unregistered_use flag I've set up instead of function
    * comparing *yech* - Dianora
    */

   if (!IsRegistered(cptr) && cmptr->unreg_use == 0) {
      sendto_one(from, ":%s %d %s %s :Register first.",
		 me.name, ERR_NOTREGISTERED, from->name, ch);
      return -1;
   }

   if (IsRegisteredUser(cptr) && cmptr->reset_idle == 1)
      from->user->last = timeofday;

   return (*cmptr->func) (cptr, from, i, para);
}
