/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_sqline.c: SQLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: s_debug.c,v 1.17 2002/10/28 21:36:14 rageircd Exp $
 */

#include "struct.h"
#include "patchlevel.h"
#include "blalloc.h"
#include "memory.h"

extern void count_whowas_memory(int *, u_long *);
extern u_long cres_mem(aClient *);
extern void count_ip_hash(int *, u_long *);

#include "numeric.h"
#include "common.h"
#include "sys.h"
#include "hash.h"
#include <sys/file.h>
#if !defined(ULTRIX) && !defined(SGI) && !defined(sequent) && !defined(__convex__)
#include <sys/param.h>
#endif
#ifdef HAVE_GETRUSAGE
#ifdef SOL20
#include <sys/time.h>
#endif
#include <sys/resource.h>
#else
#ifdef HAVE_TIMES
#include <sys/times.h>
#endif
#endif
#include "h.h"

#ifndef SYS_ERRLIST_DECLARED
extern char *sys_errlist[];
#endif

#ifdef DEBUGMODE
static char debugbuf[1024];

void debug(int level, char *pattern, ...)
{
	va_list vl;
	int err = errno;

	va_start(vl, pattern);
	ircvsprintf(debugbuf, pattern, vl);
	va_end(vl);

	if (level == DEBUG_ERROR) {
		ircdlog(LOG_ERROR, debugbuf);
	}
	if (debug_level >= 0 && level <= debug_level) {
		if (local[2] != NULL) {
			local[2]->sendM++;
			local[2]->sendB += strlen(debugbuf);
		}
		fprintf(stderr, "%s", debugbuf);
		fputc('\n', stderr);
	}
	errno = err;
}

#ifdef HAVE_GETRUSAGE
#ifdef hz
#define hzz hz
#else
#ifdef HZ
#define hzz HZ
#else
int hzz = 1;
#endif
#endif
#endif /* HAVE_GETRUSAGE */

void send_usage(aClient *cptr, char *nick)
{
#ifdef HAVE_GETRUSAGE
	struct rusage rus;
	time_t secs, rup;

	if (getrusage(RUSAGE_SELF, &rus) == -1) {
		sendto_one(cptr, ":%s NOTICE %s :Getrusage error: %s.", me.name, nick, sys_errlist[errno]);
		return;
	}

	secs = rus.ru_utime.tv_sec + rus.ru_stime.tv_sec;
	rup = timeofday - me.since;

	if (secs == 0) {
		secs = 1;
	}
	sendto_one(cptr, ":%s %d %s :CPU Secs %d:%d User %d:%d System %d:%d", me.name, RPL_STATSDEBUG, nick,
		secs / 60, secs % 60, rus.ru_utime.tv_sec / 60, rus.ru_utime.tv_sec % 60,
		rus.ru_stime.tv_sec / 60, rus.ru_stime.tv_sec % 60);
	sendto_one(cptr, ":%s %d %s :RSS %d ShMem %d Data %d Stack %d", me.name, RPL_STATSDEBUG, nick, rus.ru_maxrss,
		rus.ru_ixrss / (rup * hzz), rus.ru_idrss / (rup * hzz), rus.ru_isrss / (rup * hzz));
	sendto_one(cptr, ":%s %d %s :Swaps %d Reclaims %d Faults %d",me.name, RPL_STATSDEBUG, nick, rus.ru_nswap,
		rus.ru_minflt, rus.ru_majflt);
	sendto_one(cptr, ":%s %d %s :Block in %d out %d", me.name, RPL_STATSDEBUG, nick, rus.ru_inblock, rus.ru_oublock);
	sendto_one(cptr, ":%s %d %s :Msg Rcv %d Send %d", me.name, RPL_STATSDEBUG, nick, rus.ru_msgrcv, rus.ru_msgsnd);
	sendto_one(cptr, ":%s %d %s :Signals %d Context Vol. %d Invol %d", me.name, RPL_STATSDEBUG, nick,
		rus.ru_nsignals, rus.ru_nvcsw, rus.ru_nivcsw);
#elif defined(HAVE_TIMES)
	struct tms tmsbuf;
	int hzz = 1;
	int ticpermin = hhz * 60;
	int umin = tmsbuf.tms_utime / ticpermin;
	int smin = (tmsbuf.tms_utime % ticpermin) / (float)hzz;
	int usec = tmsbuf.tms_stime / ticpermin;
	int ssec = (tmsbuf.tms_stime % ticpermin) / (float)hzz;
	time_t secs = usec + ssec;
	time_t mins = (secs / 60) + umin + smin;

	secs %= hzz;

	if (times(&tmsbuf) == -1) {
		sendto_one(cptr, ":%s %d %s :times(2) error: %s.", me.name, RPL_STATSDEBUG, nick, strerror(errno));
		return;
	}

	secs = tmsbuf.tms_utime + tmsbuf.tms_stime;
	sendto_one(cptr, ":%s %d %s :CPU Secs %d:%d User %d:%d System %d:%d", me.name, RPL_STATSDEBUG, nick, mins,
		secs, umin, usec, smin, ssec);
#endif

	sendto_one(cptr, ":%s %d %s :Reads %d Writes %d", me.name, RPL_STATSDEBUG, nick, readcalls, writecalls);
	sendto_one(cptr, ":%s %d %s :Client DBUF alloc %d used %d", me.name, RPL_STATSDEBUG, nick, DBufCountC,
		DBufUsedCountC);
	sendto_one(cptr, ":%s %d %s :Server DBUF alloc %d used %d", me.name, RPL_STATSDEBUG, nick, DBufCountS,
		DBufUsedCountS);
	sendto_one(cptr, ":%s %d %s :Writes:  <0 %d 0 %d <16 %d <32 %d <64 %d", me.name, RPL_STATSDEBUG, nick,
		writeb[0], writeb[1], writeb[2], writeb[3], writeb[4]);
	sendto_one(cptr, ":%s %d %s :<128 %d <256 %d <512 %d <1024 %d >1024 %d", me.name, RPL_STATSDEBUG, nick,
		writeb[5], writeb[6], writeb[7], writeb[8], writeb[9]);
	return;
}

#endif

void count_memory(aClient *cptr, char *nick)
{
	return;
}
