/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/s_misc.c: misc. functions
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: s_misc.c,v 1.51 2002/11/23 21:51:06 rageircd Exp $
 */

#include <sys/time.h>
#include <time.h>
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "zlink.h"
#include "memory.h"
#include <sys/stat.h>
#include <fcntl.h>
#if !defined(ULTRIX) && !defined(SGI) && !defined(sequent) && !defined(__convex__)
#include <sys/param.h>
#endif
#if defined(AIX) || defined(SVR3)
#include <time.h>
#endif
#include "h.h"
#include "fdlist.h"

extern void conf_cleanup();
extern int user_modes[];

#ifdef ALWAYS_SEND_DURING_SPLIT
int currently_processing_netsplit = NO;
#endif

static char *months[] = {
	"January",
	"February",
	"March",
	"April",
	"May",
	"June",
	"July",
	"August",
	"September",
	"October",
	"November",
	"December"
};
static char *weekdays[] = {
	"Sunday",
	"Monday",
	"Tuesday",
	"Wednesday",
	"Thursday",
	"Friday",
	"Saturday"
};
static char buf[BUFSIZE];

static void exit_one_client(aClient *, aClient *, aClient *, char *);

struct stats ircst, *ircstp = &ircst;

char *date(time_t clock)
{
	static char buf[80], plus;
	struct tm *lt, *gm, gmbuf;
	int minswest;

	if (clock == 0) {
		time(&clock);
	}
	gm = gmtime(&clock);
	memcpy((char *)&gmbuf, (char *)gm, sizeof(gmbuf));
	gm = &gmbuf;
	lt = localtime(&clock);

	if (lt->tm_yday == gm->tm_yday) {
		minswest = (gm->tm_hour - lt->tm_hour) * 60 + (gm->tm_min - lt->tm_min);
	}
	else if (lt->tm_yday > gm->tm_yday) {
		minswest = (gm->tm_hour - (lt->tm_hour + 24)) * 60;
	}
	else {
		minswest = ((gm->tm_hour + 24) - lt->tm_hour) * 60;
	}

	plus = (minswest > 0) ? '-' : '+';
	if (minswest < 0) {
		minswest = -minswest;
	}
	ircsprintf(buf, "%s %s %d %04d -- %02d:%02d %c%02d:%02d", weekdays[lt->tm_wday], months[lt->tm_mon],
		lt->tm_mday, lt->tm_year + 1900, lt->tm_hour, lt->tm_min, plus, minswest / 60, minswest % 60);
	return buf;
}

char *smalldate(time_t clock)
{
	static char buf[MAX_DATE_STRING];
	struct tm *lt, *gm, gmbuf;

	if (clock == 0) {
		time(&clock);
	}
	gm = gmtime(&clock);
	memcpy((char *)&gmbuf, (char *)gm, sizeof(gmbuf));
	gm = &gmbuf;
	lt = localtime(&clock);
	ircsprintf(buf, "%04d/%02d/%02d %02d.%02d", lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
		lt->tm_hour, lt->tm_min);
	return buf;
}

char *myctime(time_t value)
{
	static char buf[28];
	char *p;

	strcpy(buf, ctime(&value));
	if ((p = strchr(buf, '\n')) != NULL) {
		*p = '\0';
	}
	return buf;
}

char *check_string(char *str)
{
	char *p;
	if (BadPtr(str)) {
		return "*";
	}
	for (p = str; *p != '\0'; p++) {
		if (isspace(*p)) {
			*p = '\0';
			break;
		}
	}
	return BadPtr(str) ? "*" : str;
}

char *make_nick_user_host(char *nick, char *user, char *host)
{
	static char namebuf[NICKLEN + USERLEN + HOSTLEN + 6];
	int n;
	char *t = namebuf, *s;

	for (s = check_string(nick), n = NICKLEN; *s != '\0' && n-- > 0;) {
		*t++ = *s++;
	}

	*t++ = '!';
	for (s = check_string(user), n = USERLEN; *s != '\0' && n-- > 0;) {
		*t++ = *s++;
	}

	*t++ = '@';
	for (s = check_string(host), n = HOSTLEN; *s != '\0' && n-- > 0;) {
		*t++ = *s++;
	}

	*t = '\0';
	return namebuf;
}

char *make_user_host(char *user, char *host)
{
	static char namebuf[USERLEN + HOSTLEN + 4];
	char *t = namebuf, *s;
	int n;

	for (s = check_string(user), n = USERLEN; *s != '\0' && n-- > 0;) {
		*t++ = *s++;
	}

	*t++ = '@';
	for (s = check_string(host), n = HOSTLEN; *s != '\0' && n-- > 0;) {
		*t++ = *s++;
	}

	*t = '\0';
	return namebuf;
}

char *get_client_name(aClient *sptr, int showip)
{
	static char nbuf[HOSTLEN * 2 + USERLEN + 5];

	if (MyConnect(sptr)) {
		switch (showip) {
			case TRUE:
				ircsprintf(nbuf, "%s[%s@%s]", sptr->name,
					(!(sptr->flags & FLAGS_GOTID)) ? "" : sptr->username,
					inetntoa((char *)&sptr->ip));
				break;
			case HIDEME:
				ircsprintf(nbuf, "%s[%s@0.0.0.0]", sptr->name,
					(!(sptr->flags & FLAGS_GOTID)) ? "" : sptr->username);
				break;
			default:
				if (mycmp(sptr->name, sptr->sockhost)) {
					if (USERNAMES_IN_TRACE == 1) {
						ircsprintf(nbuf, "%s[%s@%s]", sptr->name,
							sptr->user ? sptr->username : sptr->username, sptr->sockhost);
					}
					else {
						ircsprintf(nbuf, "%s[%s]", sptr->name, sptr->sockhost);
					}
				}
				else {
					return sptr->name;
				}
		}
		return nbuf;
	}
	return sptr->name;
}

void get_sockhost(aClient *cptr, char *host)
{
	char *s = NULL;
	if ((s = strchr(host, '@')) != NULL) {
		s++;
	}
	else {
		s = host;
	}
	strncpyzt(cptr->sockhost, s, sizeof(cptr->sockhost));
	return;
}

int remove_dcc_references(aClient *cptr)
{
	aClient *acptr;
	Link *lp, *tmp, **lpp, *next = NULL;
	int found;

	for (lp = cptr->user->dccallow; lp != NULL; lp = next) {
		next = lp->next;
		acptr = lp->value.cptr;
		for (found = 0, lpp = &(acptr->user->dccallow); *lpp != NULL; lpp = &((*lpp)->next)) {
			if (lp->flags == (*lpp)->flags) {
				continue;
			}
			if ((*lpp)->value.cptr == cptr) {
				if ((*lpp)->flags == DCC_LINK_ME) {
					sendto_one(acptr, ":%s %d %s :%s (%s@%s) has been removed from your "
						"DCC allow list after signing off", me.name, RPL_DCCINFO,
						acptr->name, cptr->name, cptr->username, MaskedHost(cptr));
				}
				tmp = *lpp;
				*lpp = tmp->next;
				free_slink(tmp);
				found++;
				break;
			}
		}
		if (found == 0) {
			ircdlog(LOG_ERROR, "rdr: %s was in dccallowme list[%d] of %s but not in dccallowremote list",
				acptr->name, lp->flags, cptr->name);
		}
		free_slink(lp);
	}
	return 0;
}

int remove_accept_references(aClient *cptr, int nick_change)
{
	aClient *acptr;
	Link *lp, *tmp, **lpp, *next = NULL;
	int found;

	for (lp = cptr->user->accept; lp != NULL; lp = next) {
		next = lp->next;
		acptr = lp->value.cptr;
		for (found = 0, lpp = &(acptr->user->accept); *lpp != NULL; lpp = &((*lpp)->next)) {
			if (lp->flags == (*lpp)->flags) {
				continue;
			}
			if ((*lpp)->value.cptr == cptr) {
				if ((*lpp)->flags == ACCEPT_LINK_ME) {
					sendto_one(acptr, ":%s %d %s :%s (%s@%s) has been removed from your "
						"message accept list after %s.", me.name, RPL_ACCEPTINFO,
						acptr->name, cptr->name, cptr->username, MaskedHost(cptr),
						nick_change == 1 ? "changing nicknames" : "signing off");
				}
				tmp = *lpp;
				*lpp = tmp->next;
				free_slink(tmp);
				found++;
				break;
			}
		}
		if (found == 0) {
			ircdlog(LOG_ERROR, "rar: %s was in acceptme list[%d] of %s but not in acceptremote list",
				acptr->name, lp->flags, cptr->name);
		}
		free_slink(lp);
	}
	return 0;
}

#ifdef USE_NOQUIT
void exit_one_client_in_split(aClient *cptr, aClient *dead, char *reason)
{
	chanMember *cm;
	Link *lp;

	sendto_noquit_servs_butone(0, dead, ":%s QUIT :%s", cptr->name, reason);
	sendto_common_channels(cptr, ":%s QUIT :%s", cptr->name, reason);

	while ((cm = cptr->user->channel) != NULL) {
		remove_user_from_channel(cptr, cm->chptr);
	}
	while ((lp = cptr->user->silence) != NULL) {
		del_silence(cptr, lp->value.cp);
	}
	remove_dcc_references(cptr);
	remove_accept_references(cptr, 0);
	del_from_client_hash_table(cptr->name, cptr);
	hash_check_watch(cptr, RPL_LOGOFF);
	remove_client_from_list(cptr);
	return;
}

void exit_one_server(aClient *cptr, aClient *dead, aClient *from, aClient *lcptr, char *spinfo, char *comment)
{
	aClient *acptr, *next = NULL;
	int i, j;

	Debug((DEBUG_NOTICE, "server noquit: %s", cptr->name));

	for (acptr = client; acptr; acptr = next)  {
		next = acptr->next;
		if (acptr->uplink != cptr || !IsPerson(acptr)) {
			continue;
		}
		exit_one_client_in_split(acptr, dead, spinfo);
	}

	for (acptr = client; acptr; acptr = next) {
		next = acptr->next;
		if (acptr->uplink != cptr || !IsServer(acptr)) {
			continue;
		}
		exit_one_server(acptr, dead, from, lcptr, spinfo, comment);
		next = client;
	}

	Debug((DEBUG_NOTICE, "done exiting server: %s", cptr->name));

	for (i = serv_fdlist.entry[j = 1]; j <= serv_fdlist.last_entry; i = serv_fdlist.entry[++j]) {
		if (!(acptr = local[i]) || acptr == cptr || IsMe(acptr) || acptr == dead || acptr == lcptr) {
			continue;
		}
		if (HasCapab(acptr, CAPAB_NOQUIT) && cptr != dead) {
			continue;
		}
		if (cptr->from == acptr) {
			sendto_one(acptr, ":%s SQUIT %s :%s", from->name, cptr->name, comment);
		}
		else {
			sendto_one(acptr, "SQUIT %s :%s", cptr->name, comment);
		}
	}
	del_from_client_hash_table(cptr->name, cptr); 
	hash_check_watch(cptr, RPL_LOGOFF);
	remove_client_from_list(cptr);
	return;
}

void exit_server(aClient *lcptr, aClient *cptr, aClient *from, char *comment)
{
	char splitname[HOSTLEN + HOSTLEN + 2];
	ircsprintf(splitname, "%s %s", cptr->uplink->name, cptr->name);
	Debug((DEBUG_NOTICE, "exit_server(%s, %s, %s)", cptr->name, from->name, comment));
	exit_one_server(cptr, cptr, from, lcptr, splitname, comment);
	return;
}
#endif /* USE_NOQUIT */

int exit_client(aClient *cptr, aClient *sptr, aClient *from, char *comment)
{
#ifndef USE_NOQUIT
	aClient *acptr, *nextcli;
	char comment1[HOSTLEN + HOSTLEN + 2];
#endif
	int do_cleanup = 0;

	if (MyConnect(sptr)) {
		if (sptr->flags & FLAGS_HASHED_IP) {
			remove_one_ip(sptr->ip.s_addr);
		}
		if (HasMode(sptr, UMODE_OPER)) {
			delfrom_fdlist(sptr->fd, &oper_fdlist);
		}
		if (IsClient(sptr)) {
			Count.local--;
		}
		if (sptr->serv != NULL && (sptr->serv->sflags & SFLAGS_RC4_NEG)) {
			sendto_realops("Lost server %s during negotiation: %s", sptr->name, comment);
		}
		if (sptr->class != NULL) {
			sptr->class->clients--;
		}
		if (IsServer(sptr)) {
			Count.myserver--;
			if (IsULine(sptr)) {
				Count.myulined--;
			}
			delfrom_fdlist(sptr->fd, &serv_fdlist);

			sptr->serv->conf->refcount--;
			if (sptr->serv->conf->flag.temporary == 1 && sptr->serv->conf->refcount <= 0) {
				do_cleanup = 1;
			}
		}
		if (sptr->listener != NULL && sptr->listener->bind != NULL) {
			if ((sptr->listener->bind->flag.permanent == 1 && sptr->listener->bind->clients > 1) ||
			  (sptr->listener->bind->flag.permanent == 0 && sptr->listener->bind->clients > 0)) {
				sptr->listener->bind->clients--;
			}
			if (sptr->listener->bind->flag.temporary == 1 && sptr->listener->bind->clients <= 0) {
				do_cleanup = 1;
			}
		}
		if (do_cleanup == 1) {
			conf_cleanup();
		}

		sptr->flags |= FLAGS_CLOSING;
		if (IsPerson(sptr)) {
			Link *lp, *next = NULL;
			LOpts *lopt = sptr->user->lopt;

			hash_del_watch_list(sptr);

			if (lopt != NULL) {
				for (lp = lopt->yeslist; lp != NULL; lp = next) {
					next = lp->next;
					MyFree(lp->value.cp);
					free_slink(lp);
				}
				for (lp = lopt->nolist; lp != NULL; lp = next) {
					next = lp->next;
					MyFree(lp->value.cp);
					free_slink(lp);
				}
				MyFree(sptr->user->lopt);
			}
			ircdlog(LOG_CLIENT, "disconnecting client %s", get_client_name(sptr, FALSE));
			sendto_realops_lev(CCONN_LEV, "Client exiting: %s (%s@%s) [%s] [%s]",
				sptr->name, sptr->username, sptr->host,
				(sptr->flags & FLAGS_NORMALEX) ? "Client Quit" : comment, sptr->hostip);
		}

		if (sptr->fd >= 0) {
			if (cptr != NULL && sptr != cptr) {
				sendto_one(sptr, "ERROR :Closing Link: %s %s (%s)",
					IsPerson(sptr) ? sptr->sockhost : "0.0.0.0", sptr->name, comment);
			}
			else {
				sendto_one(sptr, "ERROR :Closing Link: %s (%s)",
					IsPerson(sptr) ? sptr->sockhost : "0.0.0.0", comment);
			}
		}

		if (IsServer(sptr)) {
			sendto_ops("%s was connected for %s. %lu/%lu sendK/recvK.", sptr->name,
				secstodhms(timeofday - sptr->firsttime), sptr->sendK, sptr->receiveK);
			ircdlog(LOG_SERVER, "%s was connected for %s. %lu/%lu sendK/recvK.",
				sptr->name, secstodhms(timeofday - sptr->firsttime),
				sptr->sendK, sptr->receiveK);

			close_connection(sptr);

			sptr->sockerr = 0;
			sptr->flags |= FLAGS_DEADSOCKET;

#ifndef USE_NOQUIT
# ifdef ALWAYS_SEND_DURING_SPLIT
			currently_processing_netsplit = YES;
# endif
			strcpy(comment1, me.name);
			strcat(comment1, " ");
			strcat(comment1, sptr->name);

			for (acptr = client; acptr; acptr = nextcli) {
				nextcli = acptr->next;
				if (!IsServer(acptr) && acptr->from == sptr) {
					exit_one_client(sptr, acptr, &me, comment1);
				}
			}
			for (acptr = client; acptr; acptr = nextcli) {
				nextcli = acptr->next;
				if (IsServer(acptr) && acptr->from == sptr) {
					exit_one_client(sptr, acptr, &me, me.name);
				}
			}
# ifdef ALWAYS_SEND_DURING_SPLIT
			currently_processing_netsplit = NO;
# endif
#endif
		}
		else {
			close_connection(sptr);
			sptr->sockerr = 0;
			sptr->flags |= FLAGS_DEADSOCKET;
		}
	}
	else if (IsPerson(sptr) && !IsULine(sptr)) {
		sendto_realops_lev(GCCONN_LEV, "Client exiting at %s: %s (%s@%s) [%s]", sptr->user->server,
			sptr->name, sptr->username, sptr->host,
			(sptr->flags & FLAGS_NORMALEX) ? "Client Quit" : comment);
	}
	exit_one_client(cptr, sptr, from, comment);
	return cptr == sptr ? FLUSH_BUFFER : 0;
}

static void exit_one_client(aClient *cptr, aClient *sptr, aClient *from, char *comment)
{
#ifndef USE_NOQUIT
	aClient *acptr;
	int i;
#endif
	chanMember *cm;
	Link *lp;

	if (IsMe(sptr)) {
		sendto_ops("ERROR: tried to exit me! : %s", comment);
		return;
	}
	else if (IsServer(sptr)) {
#ifdef USE_NOQUIT
# ifdef ALWAYS_SEND_DURING_SPLIT
		currently_processing_netsplit = YES;
# endif
		exit_server(cptr, sptr, from, comment);
# ifdef ALWAYS_SEND_DURING_SPLIT
		currently_processing_netsplit = NO;
# endif
		return;
#else
		for (i = 0; i <= highest_fd; i++) {
			if (!(acptr = local[i]) || !IsServer(acptr) || acptr == cptr || IsMe(acptr)) {
				continue;
			}
			if (sptr->from == acptr) {
				sendto_one(acptr, ":%s SQUIT %s :%s", from->name, sptr->name, comment);
			}
			else {
				sendto_one(acptr, "SQUIT %s :%s", sptr->name, comment);
			}
		}
#endif /* USE_NOQUIT */
	}
	else if (!(IsPerson(sptr))) {
		/* Nothing! */
	}
	else if (*sptr->name) {
		if (!(sptr->flags & FLAGS_KILLED)) {
			sendto_serv_butone(cptr, ":%s QUIT :%s", sptr->name, comment);
		}

		send_part_to_common_channels(sptr, comment);
		send_quit_to_common_channels(sptr, comment);

		while ((cm = sptr->user->channel) != NULL) {
			remove_user_from_channel(sptr, cm->chptr);
		}
		while ((lp = sptr->user->silence) != NULL) {
			del_silence(sptr, lp->value.cp);
		}
		if (MyConnect(sptr)) {
			while ((lp = sptr->user->invited) != NULL) {
				del_invite(sptr, lp->value.chptr);
			}
		}
		remove_dcc_references(sptr);
		remove_accept_references(sptr, 0);
	}
	if (sptr->flags & FLAGS_HASHED && del_from_client_hash_table(sptr->name, sptr) != 1) {
		sendto_realops_lev(DEBUG_LEV, "HASHING ERROR! %#x !in tab %s[%s] from %#x next %#x "
			"hnext %#x prev %#x fd %d status %d user %#x", sptr, sptr->name,
			sptr->from ? sptr->from->sockhost : "??host", sptr->from, sptr->next,
			sptr->hnext, sptr->prev, sptr->fd, sptr->status, sptr->user);
		ircdlog(LOG_ERROR, "HASHING ERROR! %#x !in tab %s[%s] from %#x next %#x "
			"hnext %#x prev %#x fd %d status %d user %#x", sptr, sptr->name,
			sptr->from ? sptr->from->sockhost : "??host", sptr->from, sptr->next,
			sptr->hnext, sptr->prev, sptr->fd, sptr->status, sptr->user);
		Debug((DEBUG_ERROR, "HASHING ERROR! %#x !in tab %s[%s] from %#x next %#x "
			"hnext %#x prev %#x fd %d status %d user %#x", sptr, sptr->name,
			sptr->from ? sptr->from->sockhost : "??host", sptr->from, sptr->next,
			sptr->hnext, sptr->prev, sptr->fd, sptr->status, sptr->user));
	}
	if (IsRegistered(sptr)) {
		hash_check_watch(sptr, RPL_LOGOFF);
	}
	remove_client_from_list(sptr);
	return;
}

void init_stats()
{
	memset((char *)&ircst, '\0', sizeof(ircst));
	return;
}

void tstats(aClient *cptr, char *name)
{
	aClient *acptr;
	int i;
	struct stats tmp, *sp = &tmp;

	memcpy((char *)sp, (char *)ircstp, sizeof(*sp));

	for (i = 0; i < highest_fd; i++) {
		if (!(acptr = local[i])) {
			continue;
		}
		if (IsServer(acptr)) {
			sp->is_sbs += acptr->sendB;
			sp->is_sbr += acptr->receiveB;
			sp->is_sks += acptr->sendK;
			sp->is_skr += acptr->receiveK;
			sp->is_sti += timeofday - acptr->firsttime;
			sp->is_sv++;
			if (sp->is_sbs > 1023) {
				sp->is_sks += (sp->is_sbs >> 10);
				sp->is_sbs &= 0x3ff;
			}
			if (sp->is_sbr > 1023) {
				sp->is_skr += (sp->is_sbr >> 10);
				sp->is_sbr &= 0x3ff;
			}
		}
		else if (IsClient(acptr)) {
			sp->is_cbs += acptr->sendB;
			sp->is_cbr += acptr->receiveB;
			sp->is_cks += acptr->sendK;
			sp->is_ckr += acptr->receiveK;
			sp->is_cti += timeofday - acptr->firsttime;
			sp->is_cl++;
			if (sp->is_cbs > 1023) {
				sp->is_cks += (sp->is_cbs >> 10);
				sp->is_cbs &= 0x3ff;
			}
			if (sp->is_cbr > 1023) {
				sp->is_ckr += (sp->is_cbr >> 10);
				sp->is_cbr &= 0x3ff;
			}
		}
		else if (IsUnknown(acptr)) {
			sp->is_ni++;
		}
	}
	sendto_one(cptr, ":%s %d %s :accepts %u refused %u", me.name, RPL_STATSDEBUG, name, sp->is_ac, sp->is_ref);
	sendto_one(cptr, ":%s %d %s :unknown commands %u prefixes %u", me.name, RPL_STATSDEBUG, name, sp->is_unco,
		sp->is_unpf);
	sendto_one(cptr, ":%s %d %s :nick collisions %u unknown closes %u", me.name, RPL_STATSDEBUG, name, sp->is_kill,
		sp->is_ni);
	sendto_one(cptr, ":%s %d %s :wrong direction %u empty %u", me.name, RPL_STATSDEBUG, name, sp->is_wrdi,
		sp->is_empt);
	sendto_one(cptr, ":%s %d %s :numerics seen %u mode fakes %u", me.name, RPL_STATSDEBUG, name, sp->is_num,
		sp->is_fake);
	sendto_one(cptr, ":%s %d %s :auth successes %u fails %u", me.name, RPL_STATSDEBUG, name, sp->is_asuc,
		sp->is_abad);
	sendto_one(cptr, ":%s %d %s :local connections %u udp packets %u", me.name, RPL_STATSDEBUG, name, sp->is_loc,
		sp->is_udp);
	sendto_one(cptr, ":%s %d %s :Client Server", me.name, RPL_STATSDEBUG, name);
	sendto_one(cptr, ":%s %d %s :connected %u %u", me.name, RPL_STATSDEBUG, name, sp->is_cl, sp->is_sv);
	sendto_one(cptr, ":%s %d %s :bytes sent %u.%uK %u.%uK", me.name, RPL_STATSDEBUG, name, sp->is_cks, sp->is_cbs,
		sp->is_sks, sp->is_sbs);
	sendto_one(cptr, ":%s %d %s :bytes recv %u.%uK %u.%uK", me.name, RPL_STATSDEBUG, name, sp->is_ckr, sp->is_cbr,
		sp->is_skr, sp->is_sbr);
	sendto_one(cptr, ":%s %d %s :time connected %u %u", me.name, RPL_STATSDEBUG, name, sp->is_cti, sp->is_sti);
#ifdef FLUD
	sendto_one(cptr, ":%s %d %s :CTCP Floods Blocked %u", me.name, RPL_STATSDEBUG, name, sp->is_flud);
#endif
	return;
}

#define	_1MEG	(1024.0)
#define	_1GIG	(1024.0 * 1024.0)
#define	_1TER	(1024.0 * 1024.0 * 1024.0)
#define	_GMKs(x)	((x > _1TER) ? "Terabytes" : ((x > _1GIG) ? "Gigabytes" : \
			((x > _1MEG) ? "Megabytes" : "Kilobytes")))
#define	_GMKv(x)	((x > _1TER) ? (float)(x / _1TER) : ((x > _1GIG) ? \
			(float)(x / _1GIG) : ((x > _1MEG) ? (float)(x / _1MEG) : (float)x)))

void serv_info(aClient *cptr, char *name)
{
	static char Lformat[] = ":%s %d %s %s %u %u %u %u %u :%u %u %s";
	int i = 0, j, fd;
	long sendK = 0, receiveK = 0, uptime;
	fdlist l = serv_fdlist;
	aClient *acptr;

	for (fd = l.entry[j = 1]; j <= l.last_entry; fd = l.entry[++j]) {
		if (!(acptr = local[fd]) || (HIDEULINES == 1 && IsULine(acptr) && !HasMode(cptr, UMODE_OPER))) {
			continue;
		}
		sendK += acptr->sendK;
		receiveK += acptr->receiveK;
		sendto_one(cptr, Lformat, me.name, RPL_STATSLINKINFO, name, get_client_name(acptr, HIDEME),
			(int)DBufLength(&acptr->sendQ), (int)acptr->sendM, (int) acptr->sendK,
			(int)acptr->receiveM, (int)acptr->receiveK, timeofday - acptr->firsttime,
			timeofday - acptr->since, IsServer(acptr) ? DoesTS(acptr) ? "TS" : "Non-TS" : "-");
		if (acptr->serv->sflags & SFLAGS_ZIP_OUT) {
			unsigned long ib, ob;
			double rat;
			zip_out_get_stats(acptr->serv->zip_out, &ib, &ob, &rat);
			if (ib != 0) {
				sendto_one(cptr, ":%s %d %s : - Zip inbytes %d, outbytes %d (%3.2f%%)",
					me.name, RPL_STATSDEBUG, name, ib, ob, rat);
			}
		}
		i++;
	}
	sendto_one(cptr, ":%s %d %s :%u total server%s", me.name, RPL_STATSDEBUG, name, i, (i == 1) ? "" : "s");
	sendto_one(cptr, ":%s %d %s :Sent total : %7.2f %s", me.name, RPL_STATSDEBUG, name, _GMKv(sendK),
		_GMKs(sendK));
	sendto_one(cptr, ":%s %d %s :Recv total : %7.2f %s", me.name, RPL_STATSDEBUG, name, _GMKv(receiveK),
		_GMKs(receiveK));

	uptime = (timeofday - me.since);
	sendto_one(cptr, ":%s %d %s :Server send: %7.2f %s (%4.1f K/s)", me.name, RPL_STATSDEBUG, name,
		_GMKv(me.sendK), _GMKs(me.sendK), (float)((float)me.sendK / (float)uptime));
	sendto_one(cptr, ":%s %d %s :Server recv: %7.2f %s (%4.1f K/s)", me.name, RPL_STATSDEBUG, name,
		_GMKv(me.receiveK), _GMKs(me.receiveK), (float)((float)me.receiveK / (float)uptime));
	return;
}

void show_oper_serv_fdlists(aClient *cptr)
{
	aClient *acptr;
	int i, j = 0, fd;
	fdlist *listp = &oper_fdlist;

	sendto_one(cptr, ":%s %d %s :*** Current IRC Operators on %s ***", me.name, RPL_STATSDEBUG,
		cptr->name, me.name);
	for (fd = listp->entry[i = 1]; i <= listp->last_entry; fd = listp->entry[++i]) {
		if (!(acptr = local[fd])) {
			continue;
		}
		if (!IsClient(acptr)) {
			ircdlog(LOG_ERROR, "WARNING: non-client fd %d(%d) in oper fdlist", fd, acptr->fd);
			continue;
		}
		if (!HasMode(acptr, UMODE_OPER)) {
			delfrom_fdlist(acptr->fd, &oper_fdlist);
			continue;
		}
		sendto_one(cptr, ":%s %d %s :%s (%s@%s) [idle: %d]", me.name, RPL_STATSDEBUG, cptr->name,
			acptr->name, acptr->username, MaskedHost(acptr), timeofday - acptr->lasttime);
		j++;
	}
	sendto_one(cptr, ":%s %d %s :*** Total: %d", me.name, RPL_STATSDEBUG, cptr->name, j);

	j = 0;
	listp = &serv_fdlist;

	sendto_one(cptr, ":%s %d %s :*** Current Servers Linked to %s ***", me.name, RPL_STATSDEBUG,
		cptr->name, me.name);
	for (fd = listp->entry[i = 1]; i <= listp->last_entry; fd = listp->entry[++i]) {
		if (!(acptr = local[fd])) {
			continue;
		}
		if (!IsServer(acptr)) {
			ircdlog(LOG_ERROR, "WARNING: non-server fd %d(%d) in serv fdlist", fd, acptr->fd);
			continue;
		}
		if (IsULine(acptr) && HIDEULINES == 1 && !HasMode(cptr, UMODE_OPER)) {
			continue;
		}
		sendto_one(cptr, ":%s %d %s :%s (by: %s) [idle: %d]", me.name, RPL_STATSDEBUG,
			cptr->name, acptr->name, acptr->serv->connected_by,
			timeofday - acptr->lasttime);
		j++;
	}
	sendto_one(cptr, ":%s %d %s :*** Total: %d", me.name, RPL_STATSDEBUG, cptr->name, j);
	return;
}

/* get_mode_str - returns string of usermode flags depending on client's umodes */
char *get_mode_str(aClient *cptr)
{
	int flag, *s;
	char *m = buf;

	*m++ = '+';
	for (s = user_modes; (flag = *s) && (m - buf < BUFSIZE - 4); s += 2) {
		if (cptr->umode & flag) {
			*m++ = (char)(*(s + 1));
		}
	}
	*m = '\0';
	return buf;
}
