/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/s_user.c: user functions
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: s_user.c,v 1.126 2002/11/23 22:18:08 rageircd Exp $
 */

#include "struct.h"
#include "patchlevel.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include <sys/stat.h>
#include <utmp.h>
#include <fcntl.h>
#include "h.h"
#include "memory.h"
#include "ssl.h"
#if defined(HAVE_STRING_H)
#include <string.h>
#else
#include <strings.h>
#endif
#include "isupport.h"

static char buf[BUFSIZE];

BlockHeap *local_client_heap;
BlockHeap *remote_client_heap;
BlockHeap *local_user_heap;
BlockHeap *remote_user_heap;

char usermode_string[30];

#ifdef INCREASE_SOCK_BUFS
extern void increase_sock_bufs(int, int);
#endif
extern char chmode_string[512];

int user_modes[] = {
	UMODE_OPER,		'o',
	UMODE_INVISIBLE,	'i',
	UMODE_SNOTICE,		's',
	UMODE_KNOTICE,		'k',
	UMODE_FNOTICE,		'f',
	UMODE_DNOTICE,		'D',
	UMODE_SPAMBOT,		'm',
	UMODE_DCCNOTICE,	'e',
	UMODE_LCLICONN,		'c',
	UMODE_GCLICONN,		'G',
	UMODE_WALLOPS,		'w',
	UMODE_GLOBOPS,		'g',
	UMODE_CHATOPS,		'b',
	UMODE_REGNICK,		'r',
	UMODE_SPYING,		'y',
	UMODE_SCLIENT,		'S',
	UMODE_SADMIN,		'a',
	UMODE_NETADMIN,		'N',
	UMODE_ADMIN,		'A',
	UMODE_COADMIN,		'C',
	UMODE_HELPER,		'h',
	UMODE_MASKED,		'z',
	UMODE_PRIVACY,		'p',
	UMODE_DEAF,		'd',
	UMODE_RSTAFF,		'R',
	0,			0
};

void init_clients()
{
	local_client_heap = BlockHeapCreate((size_t)CLIENT_LOCAL_SIZE, MAXCONNECTIONS);
	remote_client_heap = BlockHeapCreate((size_t)CLIENT_REMOTE_SIZE, CLIENTS_PREALLOCATE);
	local_user_heap = BlockHeapCreate((size_t)USER_LOCAL_SIZE, MAXCONNECTIONS);
	remote_user_heap = BlockHeapCreate((size_t)USER_REMOTE_SIZE, CLIENTS_PREALLOCATE);
	return;
}

void build_usermode_string(void)
{
	int *s;
	char *m = usermode_string;

	for (s = user_modes; *s; s += 2) {
		*m++ = (char)*(s + 1);
	}
	*m = '\0';
	return;
}

aClient *next_client(aClient *next, char *ch)
{
	aClient *tmp = next;

	next = find_client(ch, tmp);
	if (tmp != NULL && tmp->prev == next) {
		return NULL;
	}
	if (next != tmp) {
		return next;
	}
	for (; next != NULL; next = next->next) {
		if (!match(ch, next->name)) {
			break;
		}
	}
	return next;
}

aClient *next_client_double(aClient *next, char *ch)
{
	aClient *tmp = next;

	next = find_client(ch, tmp);
	if (tmp != NULL && tmp->prev ==next) {
		return NULL;
	}
	if (next != tmp) {
		return next;
	}
	for (; next != NULL; next = next->next) {
		if (!match(ch, next->name) || !match(next->name, ch)) {
			break;
		}
	}
	return next;
}

int hunt_server(aClient *cptr, aClient *sptr, char *command, int server, int parc, char *parv[])
{
	aClient *acptr;
	int wilds;

	if (parc <= server || BadPtr(parv[server]) || !match(me.name, parv[server])
	  || !match(parv[server], me.name)) {
		return (HUNTED_ISME);
	}
	if ((acptr = find_client(parv[server], NULL)) != NULL) {
		if (acptr->from == sptr->from && !MyConnect(acptr)) {
			acptr = NULL;
		}
	}
	if (acptr == NULL && (acptr = find_server(parv[server], NULL)) != NULL) {
		if (acptr->from == sptr->from && !MyConnect(acptr)) {
			acptr = NULL;
		}
	}

	collapse(parv[server]);
	wilds = (strchr(parv[server], '?') || strchr(parv[server], '*'));
	if (acptr == NULL) {
		if (wilds != 0) {
			if ((acptr = find_name(parv[server], NULL)) == NULL ||
			  !IsRegistered(acptr) || !IsServer(acptr)) {
				sendto_one(sptr, err_str(ERR_NOSUCHSERVER),me.name, parv[0], parv[server]);
				return (HUNTED_NOSUCH);
			}
		}
		else {
			for (acptr = client; (acptr = next_client(acptr, parv[server])) != NULL; acptr = acptr->next) {
				if (acptr->from == sptr->from && !MyConnect(acptr)) {
					continue;
				}
				if (IsRegistered(acptr) && acptr != cptr) {
					break;
				}
			}
		}
	}
	if (acptr != NULL) {
		if (IsMe(acptr) || MyClient(acptr)) {
			return (HUNTED_ISME);
		}
		if (match(acptr->name, parv[server])) {
			parv[server] = acptr->name;
		}
		sendto_one(acptr, command, parv[0], parv[1], parv[2], parv[3], parv[4], parv[5], parv[6],
			parv[7], parv[8]);
		return (HUNTED_PASS);
	}
	sendto_one(sptr, err_str(ERR_NOSUCHSERVER), me.name, parv[0], parv[server]);
	return HUNTED_NOSUCH;
}

char *canonize(char *buffer)
{
	static char cbuf[BUFSIZE];
	char *s, *t, *cp = cbuf, *p = NULL, *p2;
	int l = 0;

	*cp = '\0';
	for (s = strtoken(&p, buffer, ","); s != NULL; s = strtoken(&p, NULL, ",")) {
		if (l > 0) {
			for (p2 = NULL, t = strtoken(&p2, cbuf, ","); t != NULL; t = strtoken(&p2, NULL, ",")) {
				if (!mycmp(s, t)) {
					break;
				}
				else if (p2 != NULL) {
					p2[-1] = ',';
				}
			}
		}
		else {
			t = NULL;
		}
		if (t == NULL) {
			if (l > 0) {
				*(cp - 1) = ',';
			}
			else {
				l = 1;
			}
			strcpy(cp, s);
			if (p != NULL) {
				cp += (p - s);
			}
		}
		else if (p2 != NULL) {
			p2[-1] = ',';
		}
	}
	return cbuf;
}

int register_user(aClient *cptr, aClient *sptr, char *nick, char *username)
{
	static char ubuf[16];
	char *parv[3], *p, tmpstr2[512];
	aClient *nsptr;
	anUser *user = sptr->user;
	int dots = 0, bad_dns = 0;
	ConfigItem_ban *ban;
	char *userhost, spamchar = '\0';

	sptr->user->last = timeofday;
	parv[0] = sptr->name;
	parv[1] = parv[2] = NULL;
	p = inetntoa((char *)&sptr->ip);
	strncpyzt(sptr->hostip, p, HOSTIPLEN + 1);

	if (MyConnect(sptr)) {
		int auth = check_client(sptr);
		char *hostptr = (*sptr->host != '\0') ? sptr->host : get_client_name(sptr, FALSE);

		if (auth != CLIENTAUTH_MATCHED && auth != CLIENTAUTH_SOCKERR) {
			ircstp->is_ref++;
		}
		switch (auth) {
			case CLIENTAUTH_NOMATCH:
				sendto_realops_lev(CCONN_LEV, CLIENTAUTH_ERRMSG, hostptr, "");
				ircdlog(LOG_CLIENT, CLIENTAUTH_ERRMSG, hostptr, "");
				return exit_client(cptr, sptr, &me, "You are not authorized to connect to "
					"this server");
				break;
			case CLIENTAUTH_SOCKERR:
				ircdlog(LOG_ERROR, "Socket error for connecting client %s", hostptr);
				return exit_client(cptr, sptr, &me, "Socket Error");
				break;
			case CLIENTAUTH_ILINEFULL:
				sendto_realops_lev(CCONN_LEV, CLIENTAUTH_ERRMSG, hostptr, " (I line is full)");
				ircdlog(LOG_CLIENT, CLIENTAUTH_ERRMSG, hostptr, " (I line is full)");
				return exit_client(cptr, sptr, &me, "Your connection class is full");
				break;
			case CLIENTAUTH_INVALIDPW:
				sendto_realops_lev(CCONN_LEV, CLIENTAUTH_ERRMSG, hostptr, " (invalid/no password)");
				ircdlog(LOG_CLIENT, CLIENTAUTH_ERRMSG, hostptr, " (invalid/no password)");
				return exit_client(cptr, sptr, &me, "Invalid/No password");
				break;
			case CLIENTAUTH_TOOMANYIPS:
				sendto_realops_lev(CCONN_LEV, CLIENTAUTH_ERRMSG, hostptr, " (too many connections)");
				ircdlog(LOG_CLIENT, CLIENTAUTH_ERRMSG, hostptr, " (too many connections)");
				return exit_client(cptr, sptr, &me, "Too many connections from your IP address");
				break;
		}
		if (ANTI_SPAMBOT_CHECKS == 1 && *(sptr->host + 1) == '\0') {
			spamchar = *sptr->host;
		}
		for (p = sptr->host; *p != '\0'; p++) {
			if (!isalnum(*p)) {
				if (*p != '-' && *p != '.') {
					bad_dns = 1;
				}
			}
			if (*p == '.') {
				dots++;
			}
		}
		if (dots == 0) {
			sendto_realops_lev(REJ_LEV, "Dropping client %s due to invalid hostname %s (contains no "
				"dots).", sptr->name, sptr->hostip);
			return exit_client(cptr, sptr, &me, "Invalid hostname");
		}
		if (bad_dns == 1) {
			sendto_one(sptr, ":%s NOTICE %s :*** You have a bad character in your hostname, using IP "
				"address instead.", me.name, sptr->name);
			strcpy(sptr->host, sptr->hostip);
			strcpy(sptr->sockhost, sptr->hostip);
		}

		if ((sptr->class->options & CLASS_IGNOREIDENTD) || (!(sptr->flags & FLAGS_GOTID) &&
		  (sptr->class->options & CLASS_NOTILDE) && !(sptr->class->options & CLASS_REQUIREIDENTD))) {
			if (sptr->username != username) {
				strncpyzt(sptr->username, username, USERLEN + 1);
			}
		}
		else if (!(sptr->flags & FLAGS_GOTID)) {
			if (sptr->class->options & CLASS_REQUIREIDENTD) {
				sendto_one(sptr, ":%s NOTICE %s :*** Your connection class requires your host to "
					"have identd installed in order to connect.", me.name, sptr->name);
				return exit_client(cptr, sptr, &me, "Your connection class requires identd");
			}
			else {
				char tmp[USERLEN + 1];
				strncpyzt(tmp, username, USERLEN + 1);
				*sptr->username = '~';
				strncpy(&sptr->username[1], tmp, USERLEN);
				sptr->username[USERLEN] = '\0';
			}
		}

		if (sptr->fd >= MAXCLIENTS + MAX_BUFFER || sptr->fd >= MAXCLIENTS - 5) {
			sendto_realops_lev(SPY_LEV, "Too many clients, rejecting %s[%s]", nick,
				sptr->sockhost);
			ircstp->is_ref++;
			return exit_client(cptr, sptr, &me, "Sorry, server is full - try later");
		}

		userhost = make_user_host(sptr->username, sptr->host);
		if ((ban = find_ban(BAN_USERHOST, userhost)) != NULL) {
			return conf_doban(cptr, sptr, ban, NULL, -1);
		}

		if (ANTI_SPAMBOT_CHECKS == 1) {
			if (drone_check_spambot(sptr, spamchar) == 1) {
				return exit_client(cptr, sptr, sptr, "Spambot detected and rejected.");
			}
		}

		{
			char *tmpstr;
			u_char c, cc = 0;
			int lower = 0, upper = 0, special = 0;

			if (strchr(sptr->username, '@') || strchr(username, '@')) {
				sendto_realops_lev(REJ_LEV, "Illegal \"@\" in username: %s (%s)",
					get_client_name(sptr, FALSE), username);
				ircstp->is_ref++;
				ircsprintf(tmpstr2, "Invalid username [%s] - '@' is not allowed!", username);
				return exit_client(cptr, sptr, sptr, tmpstr2);
			}

			if (IGNOREFIRSTCHAR == 1) {
				tmpstr = (sptr->username[0] == '~') ? &sptr->username[2] : &sptr->username[1];
				cc = (sptr->username[0] == '~') ? sptr->username[1] : sptr->username[0];

				if ((!isalnum(cc) && !strchr(" -_.", cc)) || cc > 127) {
					special++;
				}
			}
			else {
				tmpstr = (sptr->username[0] == '~') ? &sptr->username[1] : &sptr->username[0];
			}
			while (*tmpstr != '\0') {
				c = *(tmpstr++);

				if (islower(c)) {
					lower++;
					continue;
				}
				if (isupper(c)) {
					upper++;
					continue;
				}
				if ((!isalnum(c) && !strchr(" -_.", c)) || c > 127 || c < 32) {
					special++;
				}
			}
			if ((NOMIXEDCASE == 1 && lower > 0 && upper > 0) || special > 0) {
				sendto_realops_lev(REJ_LEV, "Invalid username: %s (%s@%s)", nick,
					sptr->username, sptr->host);
				ircstp->is_ref++;
				ircsprintf(tmpstr2, "Invalid username [%s]", sptr->username);
				return exit_client(cptr, sptr, &me, tmpstr2);
			}

			lower = upper = special = cc = 0;
			if (irccmp(sptr->username, username)) {
				if (IGNOREFIRSTCHAR == 1) {
					tmpstr = (username[0] == '~') ? &username[2] : &username[1];
					cc = (username[0] == '~') ? username[1] : username[0];
					if ((!isalnum(cc) && !strchr(" -_.", cc)) || cc > 127) {
						special++;
					}
				}
				else {
					tmpstr = (username[0] == '~') ? &username[1] : username;
				}
				while (*tmpstr != '\0') {
					c = *(tmpstr++);
					if (islower(c)) {
						lower++;
						continue;
					}
					if (isupper(c)) {
						upper++;
						continue;
					}
					if ((!isalnum(c) && !strchr(" -_.", c)) || c > 127) {
						special++;
					}
				}
				if ((NOMIXEDCASE == 1 && lower > 0 && upper > 0) || special > 0) {
					sendto_realops_lev(REJ_LEV, "Invalid username: %s (%s@%s)", nick,
						username, sptr->host);
					ircstp->is_ref++;
					ircsprintf(tmpstr2, "Invalid username [%s]", username);
					return exit_client(cptr, sptr, &me, tmpstr2);
				}
			}
		}

		if (sptr->username[1] == '\0' && !isalpha(sptr->username[0])) {
			sendto_realops_lev(REJ_LEV, "Invalid username: %s (%s@%s)", nick,
				sptr->username, sptr->host);
			ircstp->is_ref++;
			ircsprintf(tmpstr2, "Invalid username [%s]", sptr->username);
			return exit_client(cptr, sptr, &me, tmpstr2);
		}

		if (ANTI_ACEBOT_CHECKS == 1) {
			if (drone_check_acebot(sptr) == 1) {
				return exit_client(cptr, sptr, &me, "You match the pattern of a known "
					"trojan, please check your system.");
			}
		}

		ircdlog(LOG_CLIENT, "client connecting on port %d: %s", sptr->listener->port,
			get_client_name(sptr, FALSE));
		sendto_realops_lev(CCONN_LEV, "Client connecting: %s (%s@%s) [%s] {%s}", nick,
			sptr->username, sptr->host, sptr->hostip,
			sptr->class ? sptr->class->name : "no class");

		if (++Count.local > Count.max_loc) {
			Count.max_loc = Count.local;
			if (!(Count.max_loc % 10)) {
				sendto_ops("New max local clients: %d", Count.max_loc);
			}
		}
		if (timeofday - Count.day > 86400) {
			Count.today = 0;
			Count.day = timeofday;
		}
		if (timeofday - Count.week > 604800) {
			Count.weekly = 0;
			Count.week = timeofday;
		}
		if (timeofday - Count.month > 2592000) {
			Count.monthly = 0;
			Count.month = timeofday;
		}
		if (timeofday - Count.year > 31536000) {
			Count.yearly = 0;
			Count.year = timeofday;
		}
		Count.today++;
		Count.weekly++;
		Count.monthly++;
		Count.yearly++;
	}
	else {
		strncpyzt(sptr->username, username, USERLEN + 1);
	}

	SetClient(sptr);
	mask_client(sptr, maskme(sptr));

	if (++Count.total > Count.max_tot) {
		Count.max_tot = Count.total;
	}
	if (HasMode(sptr, UMODE_INVISIBLE)) {
		Count.invisi++;
	}

	if (MyConnect(sptr)) {
		sptr->pingval = sptr->class->pingtime;
		sptr->sendqlen = sptr->class->sendq;
		Count.cli_restart++;

#ifdef INCREASE_SOCK_BUFS
		increase_sock_bufs(sptr->fd, 0);
#endif

		sendto_one(sptr, rpl_str(RPL_WELCOME), me.name, nick, IRCNETWORK, nick,
			sptr->username, sptr->host);
		sendto_one(sptr, rpl_str(RPL_YOURHOST), me.name, nick, me.name,
			sptr->listener->ip, sptr->listener->port, ircd_version);
#ifdef IRCII_KLUDGE
		sendto_one(sptr, "NOTICE %s :*** Your host is %s, running version %s", nick,
			get_client_name(&me, TRUE), ircd_version);
#endif
		sendto_one(sptr, rpl_str(RPL_CREATED), me.name, nick, creation);
		if (sptr->listener->options & BIND_CRJAVA) {
			sendto_one(sptr, ":%s 004 %s %s CR1.8.03-%s %s %s", me.name, nick, me.name,
				ircd_version, usermode_string, chmode_string);
		}
		else {
			sendto_one(sptr, rpl_str(RPL_MYINFO), me.name, nick, me.name, ircd_version,
				usermode_string, chmode_string);
		}
		sendto_one(sptr, rpl_str(RPL_ISUPPORT), me.name, nick, isupport_one);
		sendto_one(sptr, rpl_str(RPL_ISUPPORT), me.name, nick, isupport_two);
#ifdef HAVE_OPENSSL_ON
		if (cptr->ssl != NULL && cptr->flags & FLAGS_SECURE) {
			sendto_one(sptr, "NOTICE %s :*** You are connected via %s.", nick,
				ssl_get_cipher_info(cptr->ssl));
		}
#endif
		send_lusers(sptr, sptr, 1, parv);
		if (SHORTMOTD == 1) {
			sendto_one(sptr, ":%s NOTICE %s :*** Please read the full motd if you haven't read it.",
				me.name, sptr->name);
			sendto_one(sptr, rpl_str(RPL_MOTDSTART), me.name, parv[0], me.name);
			sendto_one(sptr, rpl_str(RPL_MOTD), me.name, parv[0], "*** This is the short motd ***");
			sendto_one(sptr, rpl_str(RPL_ENDOFMOTD), me.name, parv[0]);
		}
		else {
			send_message_file(sptr, &motd);
		}
		if (!BadPtr(PROXYMONITOR_HOST) && !BadPtr(PROXYMONITOR_URL)) {
			sendto_one(sptr, ":%s NOTICE %s :*** This server runs an open proxy monitor to prevent abuse.",
				me.name, sptr->name);
			sendto_one(sptr, ":%s NOTICE %s :*** If you see connections on various ports from %s",
				me.name, sptr->name, PROXYMONITOR_HOST);
			sendto_one(sptr, ":%s NOTICE %s :*** please disregard them, as they are the monitor in action.",
				me.name, sptr->name, PROXYMONITOR_HOST);
			sendto_one(sptr, ":%s NOTICE %s :*** For more information, please visit %s.", me.name,
				sptr->name, PROXYMONITOR_URL);
		}
	}
	else if (IsServer(cptr)) {
		aClient *acptr;
		if ((acptr = find_server(user->server, NULL)) == NULL) {
			sendto_one(cptr, ":%s KILL %s :%s GHOST (no server %s on this net)", me.name,
				sptr->name, me.name, user->server);
			sendto_realops_lev(DEBUG_LEV, "No server %s for user %s[%s@%s] from %s",
				user->server, sptr->name, sptr->username, sptr->host,
				sptr->from->name);
			sptr->flags |= FLAGS_KILLED;
			return exit_client(sptr, sptr, &me, "Ghosted Client");
		}
		else if (acptr && acptr->from != sptr->from) {
			sendto_realops_lev(DEBUG_LEV, "Bad User [%s] :%s USER %s@%s %s, != %s[%s]",
				cptr->name, nick, sptr->username, sptr->host, user->server,
				acptr->name, acptr->from->name);
			sendto_one(sptr, ":%s KILL %s :%s (%s != %s[%s] USER from wrong direction)",
				me.name, sptr->name, me.name, user->server, acptr->from->name,
				acptr->from->sockhost);
			sptr->flags |= FLAGS_KILLED;
			return exit_client(sptr, sptr, &me, "USER server wrong direction");
		}
		if (!IsULine(sptr) && !(cptr->serv->sflags & SFLAGS_BURST)) {
			sendto_realops_lev(GCCONN_LEV, "Client connecting at %s: %s (%s@%s)",
				sptr->user->server, sptr->name, sptr->username, sptr->host);
		}
	}

	send_umode(NULL, sptr, 0, SEND_UMODES, ubuf);
	if (*ubuf == '\0') {
		ubuf[0] = '+';
		ubuf[1] = '\0';
	}

	hash_check_watch(sptr, RPL_LOGON);

	sendto_nickip_servs_butone(0, cptr, "NICK %s %d %ld %s %s %s %s %lu :%s", nick, sptr->hopcount + 1,
		sptr->tsinfo, ubuf, sptr->username, sptr->host, user->server, sptr->user->servicestamp,
		sptr->info);
	sendto_nickip_servs_butone(1, cptr, "NICK %s %d %ld %s %s %s %s %lu %lu :%s", nick,
		sptr->hopcount + 1, sptr->tsinfo, ubuf, sptr->username, sptr->host, user->server,
		sptr->user->servicestamp, htonl(sptr->ip.s_addr), sptr->info);

	if (MyClient(sptr)) {
		if ((nsptr = find_person(NICKSERV, NULL)) != NULL && *sptr->passwd != '\0') {
			sendto_one(nsptr, ":%s PRIVMSG %s@%s :IDENTIFY %s", sptr->name, NICKSERV,
				SERVICES_SERVER, sptr->passwd);
		}
		memset(sptr->passwd, '\0', PASSWDLEN);
		if (ubuf[1] != '\0') {
			send_umode(cptr, sptr, 0, ALL_UMODES, ubuf);
		}
		if (!BadPtr(AUTOJOINCHANS)) {
			char *chans[3] = { sptr->name, AUTOJOINCHANS, NULL };
			runHook4(HOOKTYPE_JOIN, sptr, sptr, 3, chans);
		}
	}
	return 0;
}

int do_user(char *nick, aClient *cptr, aClient *sptr, char *username, char *host,
  char *server, unsigned long serviceid, unsigned int nickip, char *realname)
{
	anUser *user = make_user(sptr);

	if (!MyConnect(sptr)) {
		user->server = find_or_add(server);
		strncpyzt(sptr->host, host, HOSTLEN + 1);
	}
	else {
		if (!IsUnknown(sptr)) {
			sendto_one(sptr, rpl_str(ERR_ALREADYREGISTRED), me.name, nick);
			return 0;
		}
		AddMode(sptr, MODESONCONNECT);
		user->server = me.name;
	}

	strncpyzt(sptr->info, realname, sizeof(sptr->info));
	sptr->user->servicestamp = serviceid;

	if (!MyConnect(sptr)) {
		sptr->ip.s_addr = ntohl(nickip);
	}
	else {
		sptr->oflag = 0;
	}
	if (*sptr->name != '\0') {
		return register_user(cptr, sptr, sptr->name, username);
	}
	else {
		strncpyzt(sptr->username, username, USERLEN + 1);
	}
	return 0;
}

/*
 * user_mode
 *	parv[0] = sender prefix
 *	parv[1] = client to change mode for
 *	parv[2] = modes to change
 */
int user_mode(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int flag, *s, setflags = 0, what = MODE_ADD;
	char *m, **p;
	aClient *acptr;
	short badflag = 0;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MODE");
		return 0;
	}
	if ((acptr = find_person(parv[1], NULL)) == NULL) {
		if (MyConnect(sptr)) {
			sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0],
				parv[1]);
		}
		return 0;
	}
	if (IsServer(sptr) || sptr != acptr || acptr->from != sptr->from) {
		if (!IsServer(cptr)) {
			sendto_one(sptr, err_str(ERR_USERSDONTMATCH), me.name, parv[0]);
		}
		return 0;
	}
	if (parc < 3) {
		m = buf;
		*m++ = '+';
		for (s = user_modes; (flag = *s) && (m - buf < BUFSIZE - 4); s += 2) {
			if (sptr->umode & flag) {
				*m++ = (char)(*(s + 1));
			}
		}
		*m = '\0';
		sendto_one(sptr, rpl_str(RPL_UMODEIS), me.name, parv[0], buf);
		return 0;
	}
	for (s = user_modes; (flag = *s); s += 2) {
		if (sptr->umode & flag) {
			setflags |= flag;
		}
	}
	for (p = &parv[2]; p && *p; p++) {
		for (m = *p; *m; m++) {
			switch (*m) {
				case '+':
					what = MODE_ADD;
					break;
				case '-':
					what = MODE_DEL;
					break;
				case ' ':
				case '\n':
				case '\r':
				case '\t':
				case 'r':
					break;
				default:
					for (s = user_modes; (flag = *s); s += 2) {
						if (*m == (char)(*(s + 1))) {
							if (what == MODE_ADD) {
								sptr->umode |= flag;
							}
							else {
								sptr->umode &= ~flag;
							}
							break;
						}
					}
					if (flag == 0 && MyConnect(sptr)) {
						badflag = 1;
					}
					break;
			}
		}
	}
	if (badflag == 1) {
		sendto_one(sptr, err_str(ERR_UMODEUNKNOWNFLAG), me.name, parv[0]);
	}

	if (!(setflags & UMODE_OPER) && HasMode(sptr, UMODE_OPER) && !IsServer(cptr)) {
		DelMode(sptr, UMODE_OPER);
	}
	if (!(setflags & UMODE_OPER) && HasMode(sptr, UMODE_OPER)) {
		Count.oper++;
	}
	if ((setflags & UMODE_OPER) && !HasMode(sptr, UMODE_OPER)) {
		Count.oper--;
		if (MyConnect(sptr)) {
			delfrom_fdlist(sptr->fd, &oper_fdlist);
			sptr->oflag = 0;
		}
	}
	if (!(setflags & UMODE_INVISIBLE) && HasMode(sptr, UMODE_INVISIBLE)) {
		Count.invisi++;
	}
	if ((setflags & UMODE_INVISIBLE) && !HasMode(sptr, UMODE_INVISIBLE)) {
		Count.invisi--;
	}
	if ((setflags & UMODE_MASKED) && !HasMode(sptr, UMODE_MASKED)) {
		if (sptr->user->uflags & UFLAGS_VHOST) {
			sptr->user->uflags &= ~UFLAGS_VHOST;
		}
	}
	if (!HasMode(sptr, UMODE_OPER) && !IsServer(cptr)) {
		DelMode(sptr, UMODE_FNOTICE|UMODE_DNOTICE|UMODE_SPAMBOT|UMODE_DCCNOTICE|UMODE_LCLICONN);
		DelMode(sptr, UMODE_GCLICONN|UMODE_GLOBOPS|UMODE_CHATOPS|UMODE_SPYING|UMODE_SADMIN);
		DelMode(sptr, UMODE_NETADMIN|UMODE_ADMIN|UMODE_COADMIN|UMODE_HELPER|UMODE_RSTAFF);
	}
	if (!IsULine(sptr) && !IsServer(cptr)) {
		if (!(setflags & UMODE_SCLIENT)) {
			DelMode(sptr, UMODE_SCLIENT);
		}
		if (!(setflags & UMODE_DEAF)) {
			DelMode(sptr, UMODE_DEAF);
		}
	}
	if (MyClient(sptr) && HasMode(sptr, UMODE_OPER)) {
		if (HasMode(sptr, UMODE_SADMIN) && !OPHasFlag(sptr, OFLAG_SADMIN)) {
			DelMode(sptr, UMODE_SADMIN);
		}
		if (HasMode(sptr, UMODE_NETADMIN) && !OPHasFlag(sptr, OFLAG_NETADMIN)) {
			DelMode(sptr, UMODE_NETADMIN);
		}
		if (HasMode(sptr, UMODE_ADMIN) && !OPHasFlag(sptr, OFLAG_ADMIN)) {
			DelMode(sptr, UMODE_ADMIN);
		}
		if (HasMode(sptr, UMODE_COADMIN) && !OPHasFlag(sptr, OFLAG_COADMIN)) {
			DelMode(sptr, UMODE_COADMIN);
		}
		if (HasMode(sptr, UMODE_FNOTICE) && !OPHasFlag(sptr, OFLAG_FNOTICE)) {
			DelMode(sptr, UMODE_FNOTICE);
		}
		if (HasMode(sptr, UMODE_RSTAFF) && !OPHasFlag(sptr, OFLAG_RSTAFF)) {
			DelMode(sptr, UMODE_RSTAFF);
		}
		if (HasMode(sptr, UMODE_LCLICONN) && !OPHasFlag(sptr, OFLAG_LCLICONN)) {
			DelMode(sptr, UMODE_LCLICONN);
		}
		if (HasMode(sptr, UMODE_GCLICONN) && !OPHasFlag(sptr, OFLAG_GCLICONN)) {
			DelMode(sptr, UMODE_GCLICONN);
		}
	}
	mask_client(sptr, maskme(sptr));
	send_umode_out(cptr, sptr, setflags);
	return 0;
}

void send_umode(aClient *cptr, aClient *sptr, int old, int sendmask, char *umode_buf)
{
	int *s, flag, what = 0;
	char *m = umode_buf;

	*m = '\0';
	for (s = user_modes; (flag = *s); s += 2) {
		if (MyClient(sptr) && !(flag & sendmask)) {
			continue;
		}
		if ((flag & old) && !(sptr->umode & flag)) {
			if (what == MODE_DEL) {
				*m++ = *(s + 1);
			}
			else {
				what = MODE_DEL;
				*m++ = '-';
				*m++ = *(s + 1);
			}
		}
		else if (!(flag & old) && (sptr->umode & flag)) {
			if (what == MODE_ADD) {
				*m++ = *(s + 1);
			}
			else {
				what = MODE_ADD;
				*m++ = '+';
				*m++ = *(s + 1);
			}
		}
	}
	*m = '\0';
	if (*umode_buf != '\0' && cptr != NULL) {
		sendto_one(cptr, ":%s MODE %s :%s", sptr->name, sptr->name, umode_buf);
	}
	return;
}

void send_umode_out(aClient *cptr, aClient *sptr, int old)
{
	int i, j;
	aClient *acptr;
	fdlist listp = serv_fdlist;

	send_umode(NULL, sptr, old, SEND_UMODES, buf);
	for (i = listp.entry[j = 1]; j <= listp.last_entry; i = listp.entry[++j]) {
		if ((acptr = local[i]) && acptr != cptr && acptr != sptr && *buf) {
			sendto_one(acptr, ":%s MODE %s :%s", sptr->name, sptr->name, buf);
		}
	}
	if (cptr != NULL && MyClient(cptr)) {
		send_umode(cptr, sptr, old, ALL_UMODES, buf);
	}
	return;
}

int is_silenced(aClient *sptr, aClient *acptr)
{
	Link *lp;
	char *sender;

	if (acptr->user == NULL || acptr->user->silence == NULL || sptr->user == NULL) {
		return 0;
	}

	sender = make_nick_user_host(sptr->name, sptr->username, sptr->host);
	for (lp = acptr->user->silence; lp != NULL; lp = lp->next) {
		if (!match(lp->value.cp, sender)) {
			if (!MyConnect(sptr)) {
				sendto_one(sptr->from, ":%s SILENCE %s :%s", acptr->name, sptr->name,
					lp->value.cp);
				lp->flags = 1;
			}
			return 1;
		}
	}
	return 0;
}

int del_silence(aClient *sptr, char *mask)
{
	Link **lp, *tmp;
	for (lp = &(sptr->user->silence); *lp != NULL; lp = &((*lp)->next)) {
		if (!mycmp(mask, (*lp)->value.cp)) {
			tmp = *lp;
			*lp = tmp->next;
			MyFree(tmp->value.cp);
			free_slink(tmp);
			return 0;
		}
	}
	return 1;
}

int add_silence(aClient *sptr, char *mask)
{
	Link *lp;
	int cnt = 0, len = 0;

	for (lp = sptr->user->silence; lp !=  NULL; lp = lp->next) {
		len += strlen(lp->value.cp);
		if (MyClient(sptr)) {
			if (len > MAXSILELENGTH || ++cnt >= MAXSILES) {
				sendto_one(sptr, err_str(ERR_SILELISTFULL), me.name, sptr->name, mask);
				return -1;
			}
			else {
				if (!match(lp->value.cp, mask)) {
					return -1;
				}
			}
		}
		else if (!mycmp(lp->value.cp, mask)) {
			return -1;
		}
	}
	lp = make_slink();
	lp->next = sptr->user->silence;
	lp->value.cp = (char *)MyMalloc(strlen(mask) + 1);
	strcpy(lp->value.cp, mask);
	sptr->user->silence = lp;
	return 0;
}

void expire_kline(void *b)
{
	ConfigItem_ban *ban = (ConfigItem_ban *)b, *ban2;

	if (ban == NULL) {
		return;
	}
	for (ban2 = ci_ban; ban2 != NULL; ban2 = (ConfigItem_ban *)ban2->next) {
		if (ban == ban2) {
			sendto_realops("Timed K:Line on %s has now expired.", ban2->mask);
			del_ConfigItem((ConfigItem *)ban2, (ConfigItem **)&ci_ban);
			MyFree(ban2->mask);
			MyFree(ban2->reason);
			MyFree(ban2);
			break;
		}
	}
	return;
}
